package com.ibm.commerce.sap.jca.commands;

/*
 *-------------------------------------------------------------------
 * Licensed Materials - Property of IBM
 *
 * 5724-A18
 *
 * (c) Copyright International Business Machines Corporation. yyyy, 2003
 *     All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 *-------------------------------------------------------------------
 */

//*---------------------------------------------------------------------
//* The sample contained herein is provided to you "AS IS".
//*
//* It is furnished by IBM as a simple example and has not been  
//* thoroughly tested
//* under all conditions.  IBM, therefore, cannot guarantee its 
//* reliability, serviceability or functionality.  
//*
//* This sample may include the names of individuals, companies, brands 
//* and products in order to illustrate concepts as completely as 
//* possible.  All of these names
//* are fictitious and any similarity to the names and addresses used by 
//* actual persons 
//* or business enterprises is entirely coincidental.
//*---------------------------------------------------------------------
//*


////////////////////////////////////////////////////////////////////////////////
//
// Change History:
//
// YYMMDD 	WHO    Description
// --------------------------------------------------------------------
// 020527  	Rajesh Danda
//
////////////////////////////////////////////////////////////////////////////////


import com.ibm.commerce.taxation.objects.*;
import com.ibm.commerce.datatype.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.usermanagement.commands.*;
import com.ibm.commerce.util.ResourceBundleHandler;

import javax.naming.Context;
import java.util.*;

import com.ibm.commerce.sap.jca.util.*;
import com.ibm.eis.sap.jca.generated.*;



/**
 * The implementation class for AddSAPOrgEntityCmd.
 * This is used for registering a customer in SAP,
 * which is treated as equivalent to organization in WC
 * This is called during PostOrg Entity registration.
 * This command registers only organization and not the organization units
 *
 */
 
public class AddSAPOrgEntityCmdImpl extends TaskCommandImpl implements AddSAPOrgEntityCmd
{
	public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
	public final static String CLASSNAME = "com.ibm.commerce.sap.jca.commands.AddSAPOrgEntityCmdImpl";
	protected TypedProperty requestProperties = null;
	protected TypedProperty responseProperties = null;

	CustomerCreateBapi cmd = new CustomerCreateBapi();

public TypedProperty getRequestProperties()
{
	return requestProperties;
}
public TypedProperty getResponseProperties()
{
	return responseProperties;
}
/**
 * Establishing the connection to SAP backend system
 */
protected void initialize() throws ECException {
	
    final String METHODNAME = "initialize";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);


    try{


	    // Setting properties at runtime

            Properties configProp = new Properties();

            configProp.put(
                Context.INITIAL_CONTEXT_FACTORY,               
                PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._INITIAL_CONTEXT_FACTORY));
            configProp.put(Context.PROVIDER_URL, PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._PROVIDER_URL));

            Context ctx = null;
            ctx = new javax.naming.InitialContext(configProp);

			String jndiResource = PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._JNDI_RESOURCE);
	
            com.ibm.connector2.sap.SAPConnectionFactory scf =
                (com.ibm.connector2.sap.SAPConnectionFactory) ctx.lookup(jndiResource);

            com.ibm.connector2.sap.SAPConnectionSpec scs =
                new com.ibm.connector2.sap.SAPConnectionSpec();

  				scs.setUserName(PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._USERID));
				scs.setPassword(PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._PASSWD));
				 
            com.ibm.connector2.sap.SAPConnection connection =
                (com.ibm.connector2.sap.SAPConnection) scf.getConnection(scs);
            com.ibm.connector2.sap.SAPInteractionSpec ispec =
                new com.ibm.connector2.sap.SAPInteractionSpec();
            com.ibm.connector2.sap.SAPInteraction interaction =
                (com.ibm.connector2.sap.SAPInteraction) connection.createInteraction();

            cmd.setCeInteractionSpec(ispec);
            cmd.setInteraction(interaction);


     } catch (javax.resource.ResourceException e) {
        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASSNAME,
            METHODNAME, 
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);        
    } catch (javax.naming.NamingException e) {
        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,true);
    } catch (Exception e) {
        throw new ECSystemException(ECMessage._ERR_GENERIC, CLASSNAME, METHODNAME, e,true);
    }

    ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);
  
	}
/**
 * Contains the actual business logic of the command
 */

public void performExecute() throws ECException {

    //Add the business logic here	
    final String METHODNAME = "performExecute";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);

    boolean executed = false;
    String newOrgEntityID = null;
    String refNumber = null;

    initialize();

    try {

        //  Create BAPI_CUSTOMER_CREATEFROMDATA
        BAPI_CUSTOMER_CREATEFROMDATA custCreate = new BAPI_CUSTOMER_CREATEFROMDATA();

        // set PI_ADDRESS
        com.ibm.eis.sap.jca.generated.Bapikna101Structure piAddr =
            new com.ibm.eis.sap.jca.generated.Bapikna101Structure();

        //orgentity name is mapped to last name    
        String orgEntityName =
            requestProperties.getString(ECUserConstants.EC_ORG_ORGENTITYNAME);
        piAddr.setName(orgEntityName.trim());

        // need to take care of street length into SAP
        String street1 =
            requestProperties.getString(ECUserConstants.EC_ADDR_ADDRESS1, null);
        String street2 =
            requestProperties.getString(ECUserConstants.EC_ADDR_ADDRESS2, null);
        String street3 =
            requestProperties.getString(ECUserConstants.EC_ADDR_ADDRESS3, null);

        StringBuffer street = new StringBuffer();

        if ((street1 != null) && (!street1.trim().equals("")))
            street.append(street1 + ",");
        if ((street2 != null) && (!street2.trim().equals("")))
            street.append(street2 + ",");
        if ((street3 != null) && (!street3.trim().equals("")))
            street.append(street3);

        // take care of datalength mismatch between the 2 systems		  

        if (street.length() > 35) {
            piAddr.setStreet(street.substring(0, 34));
        } else {
            piAddr.setStreet(street.toString());
        }

        String postalcode =
            requestProperties.getString(ECUserConstants.EC_ADDR_ZIPCODE);
        piAddr.setPostlCode(postalcode);

        String city = requestProperties.getString(ECUserConstants.EC_ADDR_CITY);
        piAddr.setCity(city);

        String lang = "-1"; //lang is not provided for organizations

        String altLang = "SAPLANG_" + lang.substring(1, lang.length());
        piAddr.setLangu(PropertiesKeysForSAP.sapResourceBundle.getString(altLang));

        String stateAbbr =
            requestProperties.getString(ECUserConstants.EC_ADDR_STATE, null);

        //to get SAP statecode for a given wc state(s)                  
        StateCodeAccessBean stateCode = new StateCodeAccessBean();
        stateCode.setInitKey_strStateAbbr(stateAbbr);
        stateCode.setInitKey_strProductVendor(
            PropertiesKeysForSAP.sapResourceBundle.getString(
                PropertiesKeysForSAP._PRODUCT_VENDOR));
        stateCode.refreshCopyHelper();
        piAddr.setRegion(stateCode.getStateCode());

        String country =
            requestProperties.getString(ECUserConstants.EC_ADDR_COUNTRY, null);
        piAddr.setCountry(country);

        custCreate.setPiAddress(piAddr);

        // set PI_COPYREFERENCE
        com.ibm.eis.sap.jca.generated.Bapikna102Structure piCopyRef =
            new com.ibm.eis.sap.jca.generated.Bapikna102Structure();

        piCopyRef.setSalesorg(
            PropertiesKeysForSAP.sapResourceBundle.getString(
                PropertiesKeysForSAP._SALES_ORGANIZATION));
        piCopyRef.setDistrChan(
            PropertiesKeysForSAP.sapResourceBundle.getString(
                PropertiesKeysForSAP._DISTRIBUTION_CHANNEL));
        piCopyRef.setDivision(
            PropertiesKeysForSAP.sapResourceBundle.getString(
                PropertiesKeysForSAP._DIVISION));

        // get the parent Reference number
        // registering Organizations alone, and not Organization Units.

        String parentID =
            requestProperties.getString(ECUserConstants.EC_PARENTMEMBERID, null);
        if ((parentID == null) || (parentID.trim().equals(""))) {
            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASSNAME,
                METHODNAME,
                "**Parent Id is null**");
            ECTrace.exit(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASSNAME,
                METHODNAME,
                "Parent Id");
            throw new ECApplicationException(
                ECMessageObjectsForSAP._ERR_CUSTOMER_CREATE,
                CLASSNAME,
                METHODNAME,
                true);
        } else {

            com.ibm.commerce.user.objects.OrganizationAccessBean accBean =
                new com.ibm.commerce.user.objects.OrganizationAccessBean();
            accBean.setInitKey_MemberId(parentID);
            accBean.refreshCopyHelper();
            piCopyRef.setRefCustmr(accBean.getOrgEntityField1());
        }

        newOrgEntityID =
            responseProperties.getString(ECUserConstants.EC_ORG_ORGENTITYID);

        custCreate.setPiCopyreference(piCopyRef);

        // Set command params
        cmd.setCeInput(custCreate);
        cmd.execute();
        executed = true;

        BapireturnStructure retStruct = cmd.getCeOutput0().getReturn();
        String retType = retStruct.getType();
        String retData = retStruct.getMessage();

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASSNAME,
            METHODNAME,
            "**SAP Return Type** " + retType);
        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASSNAME,
            METHODNAME,
            "**SAP Return Data** " + retData);

        if (retType.equals("E")) {
            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASSNAME,
                METHODNAME,
                "**Failed to create SAP Customer Record**");

            throw new ECApplicationException(
                ECMessageObjectsForSAP._ERR_CUSTOMER_CREATE,
                CLASSNAME,
                METHODNAME,
                true);

        } else {

            refNumber = cmd.getCeOutput0().getPeCustomer();

            com.ibm.commerce.user.objects.OrganizationAccessBean accBean =
                new com.ibm.commerce.user.objects.OrganizationAccessBean();
            accBean.setInitKey_MemberId(newOrgEntityID);
            accBean.refreshCopyHelper();
            accBean.setOrgEntityField1(refNumber);
            accBean.commitCopyHelper();

            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASSNAME,
                METHODNAME,
                "** Organization record created successfully**");

        }

    } catch (javax.ejb.CreateException e) {

        if (executed) {

            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASSNAME,
                METHODNAME,
                "Failed to create the buyer organization in WebSphere Commerce, WC OrganizationID = "
                    + newOrgEntityID
                    + " SAP Customer Number = "
                    + refNumber);

            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASSNAME,
                METHODNAME,
                "Action : Delete the customer in SAP by calling a BAPI to remove the customer record. If it fails, manually remove the customer record in SAP");
        }

        throw new ECSystemException(
            ECMessage._ERR_CREATE_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    } catch (javax.ejb.FinderException e) {

        if (executed) {

            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASSNAME,
                METHODNAME,
                "Failed to create the buyer organization in WebSphere Commerce, WCS OrganizationID = "
                    + newOrgEntityID
                    + " SAP Customer Number = "
                    + refNumber);

            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASSNAME,
                METHODNAME,
                "Action : Delete the customer in SAP by calling a BAPI to remove the customer record. If it fails, manually remove the customer record in SAP");
        }

        throw new ECSystemException(
            ECMessage._ERR_FINDER_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);

    } catch (javax.naming.NamingException e) {

        if (executed) {

            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASSNAME,
                METHODNAME,
                "Failed to create the buyer organization in WebSphere Commerce, WCS OrganizationID = "
                    + newOrgEntityID
                    + " SAP Customer Number = "
                    + refNumber);

            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASSNAME,
                METHODNAME,
                "Action : Delete the customer in SAP by calling a BAPI to remove the customer record. If it fails, manually remove the customer record in SAP");
        }

        throw new ECSystemException(
            ECMessage._ERR_NAMING_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    } catch (java.rmi.RemoteException e) {

        if (executed) {

            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASSNAME,
                METHODNAME,
                "Failed to create the buyer organization in WebSphere Commerce, WCS OrganizationID = "
                    + newOrgEntityID
                    + " SAP Customer Number = "
                    + refNumber);

            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASSNAME,
                METHODNAME,
                "Action : Delete the customer in SAP by calling a BAPI to remove the customer record. If it fails, manually remove the customer record in SAP");
        }

        throw new ECSystemException(
            ECMessage._ERR_REMOTE_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
       }

    ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);

}
public void setRequestProperties(TypedProperty aRequestProperties)
{
	requestProperties = aRequestProperties;
}
public void setResponseProperties(TypedProperty aResponseProperties)
{
	responseProperties = aResponseProperties;
}
}
