package com.ibm.commerce.sap.jca.commands;

/*
 *-------------------------------------------------------------------
 * Licensed Materials - Property of IBM
 *
 * 5724-A18
 *
 * (c) Copyright International Business Machines Corporation. yyyy, 2003
 *     All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 *-------------------------------------------------------------------
 */

//*---------------------------------------------------------------------
//* The sample contained herein is provided to you "AS IS".
//*
//* It is furnished by IBM as a simple example and has not been  
//* thoroughly tested
//* under all conditions.  IBM, therefore, cannot guarantee its 
//* reliability, serviceability or functionality.  
//*
//* This sample may include the names of individuals, companies, brands 
//* and products in order to illustrate concepts as completely as 
//* possible.  All of these names
//* are fictitious and any similarity to the names and addresses used by 
//* actual persons 
//* or business enterprises is entirely coincidental.
//*---------------------------------------------------------------------
//*


////////////////////////////////////////////////////////////////////////////////
//
// Change History:
//
// YYMMDD  	WHO    Description
// --------------------------------------------------------------------
// 020527  	Anand
//
////////////////////////////////////////////////////////////////////////////////


import com.ibm.commerce.inventory.objects.*;
import java.sql.*;


import com.ibm.commerce.datatype.*;
import java.util.*;
import java.rmi.*;
import javax.ejb.*;
import javax.naming.*;

import com.ibm.commerce.order.objects.*;
import com.ibm.commerce.fulfillment.commands.*;


import com.ibm.commerce.ras.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.command.*;

import com.ibm.commerce.sap.jca.util.*;
import com.ibm.commerce.util.ResourceBundleHandler;
import com.sap.rfc.*;
import com.sap.rfc.exception.*;
import com.ibm.connector2.sap.*;
import com.ibm.eis.sap.jca.generated.*;

/*
*This command checks the inventory availability details at the SAP end
* for the given list of order items. 
*
* For a given order item and a given quantity, it gets list of possible
* delivery schedules and sets them to a vector which is put into request properties
*/

public class SAPCheckInventoryAvailabilityCmdImpl  extends CheckInventoryAvailabilityCmdImpl implements CheckInventoryAvailabilityCmd
{

	public final static String CLASSNAME = "com.ibm.commerce.sap.jca.commands.SAPCheckInventoryAvailabilityCmdImpl";
	public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;

        private SAPConnectionSpec connSpec = new SAPConnectionSpec();
	private SAPConnectionFactory connFactory = null;
	private SAPConnection connection = null;

	MaterialAvailabilityBapi cmd = new MaterialAvailabilityBapi();
	
	private java.util.Enumeration ienOrderItems = null;

/*
 * Checks inventory availability for each catalog entry.
 * It invokes BAPI command to get the results
 */

private Vector checkInventoryForCatEntry(
    String partNumber,
    Integer quantity,
    Integer storeId,
    String qtyMeasure)
    throws ECException {

    final String METHODNAME = "checkInventoryForCatEntry";

    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);

    Vector vMatAvailList = new Vector();

    try {

        //retrieve the availability details for the given part number
        BAPI_MATERIAL_AVAILABILITY matAvail = new BAPI_MATERIAL_AVAILABILITY();
        BapiwmdvsTable wmdvsxTable = new BapiwmdvsTable();
        BapiwmdvsTableRow wmdvsxTableRow = wmdvsxTable.createEmptyRow();
        wmdvsxTableRow.setReqQtyString(quantity.toString()); //requested qty
        wmdvsxTable.appendRow(wmdvsxTableRow);

        matAvail.setWmdvsx(wmdvsxTable);
        matAvail.setMaterial(partNumber);
        matAvail.setPlant(
            PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._PLANT));
        matAvail.setUnit(qtyMeasure);

        cmd.setCeInput(matAvail);

        cmd.execute();


        BAPI_MATERIAL_AVAILABILITY matAvailOut = cmd.getCeOutput0();

        BapireturnStructure returnStruct = matAvailOut.getReturn();

        //set material availability information to a vector

        if (returnStruct.getType().equals("E")) {
            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASSNAME,
                METHODNAME,
                "** Error occured when material availaiblity BAPI called **");

            throw new ECApplicationException(
                ECMessageObjectsForSAP._ERR_MATERIAL_AVAILABILITY,
                CLASSNAME,
                METHODNAME,
                true);
        }

        BapiwmdveTable wmdvex = matAvailOut.getWmdvex();
        for (int i = 0; i < wmdvex.getRowCount(); i++) {
            // check whether the confirmed qty is zero, If zero, no need to include this entry
            if (wmdvex.getRow(i).getComQty().intValue() != 0) {

                Object[] availableList = { "", "", new Integer(0), new Integer(0)};
                availableList[0] = matAvailOut.getDialogflag();
                if (!matAvailOut.getDialogflag().equals("N")) {
                    availableList[1] = new Timestamp(wmdvex.getRow(i).getComDate().getTime());
                    availableList[2] = new Integer(wmdvex.getRow(i).getComQty().intValue());
                    availableList[3] = quantity;
                }
                vMatAvailList.addElement(availableList);
            }
        }

    } catch (JRfcRemoteException e) {
        throw new ECSystemException(
            ECMessageObjectsForSAP._ERR_JRFC_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);

    }

    ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);

    return vMatAvailList;
}
/**
 *  Gets items to check inventory.
 *  @return	ienOrderItems
 */
public java.util.Enumeration getOrderItems() {
	return ienOrderItems;
}
/**
 * Establishing the connection to SAP backend system
 */
 
protected void initialize() throws com.ibm.commerce.exception.ECException {
    final String METHODNAME = "initialize";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);


    try{

        // Setting properties at runtime


		
            Properties configProp = new Properties();
            configProp.put(
                Context.INITIAL_CONTEXT_FACTORY,               
                PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._INITIAL_CONTEXT_FACTORY));
            configProp.put(Context.PROVIDER_URL, PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._PROVIDER_URL));

            Context ctx = null;
            ctx = new javax.naming.InitialContext(configProp);
             		String jndiResource = PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._JNDI_RESOURCE);
            connFactory = (com.ibm.connector2.sap.SAPConnectionFactory) ctx.lookup(jndiResource);

			connSpec.setUserName(PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._USERID));
			connSpec.setPassword(PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._PASSWD));
			 
            com.ibm.connector2.sap.SAPConnection connection =
                (com.ibm.connector2.sap.SAPConnection) connFactory.getConnection(connSpec);

            com.ibm.connector2.sap.SAPInteractionSpec ispec =
                new com.ibm.connector2.sap.SAPInteractionSpec();
            com.ibm.connector2.sap.SAPInteraction interaction =
                (com.ibm.connector2.sap.SAPInteraction) connection.createInteraction();
			connection.setTraceLevel(SAPGlobals.RAS_TRACE_INTERNAL);
			interaction.setTraceLevel(SAPGlobals.RAS_TRACE_INTERNAL);
            cmd.setCeInteractionSpec(ispec);
            cmd.setInteraction(interaction);
           

     } catch (javax.resource.ResourceException e) {
        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASSNAME,
            METHODNAME, 
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);    
    } catch (javax.naming.NamingException e) {
        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,true);
    } catch (Exception e) {
        throw new ECSystemException(ECMessage._ERR_GENERIC, CLASSNAME, METHODNAME, e,true);
    }

    ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);
  
	
	}
/**
 * Contains the actual business logic of the command.
 */
public void performExecute() throws ECException {
    final String METHODNAME = "performExecute";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);

    try {

        initialize(); //initializing SAP connection

        //to add the list of orderitems for availability check
        TypedProperty orderItemsList = new TypedProperty();
        Vector vMatAvailList = new Vector();

        Vector vCheckOrderItems = new Vector();

        //Get the list of selected orderitems only	
        if (ienOrderItems != null) { //

            //String strCheckOrderItems =  ienOrderItems.toString();

            int distinctOrderItemCount = 0;
            //for getting the number of items from orderItemsList property

            while (ienOrderItems.hasMoreElements()) {
                OrderItemAccessBean abOrderItem =
                    (OrderItemAccessBean) ienOrderItems.nextElement();

                //if new item, put the values into Hashtable, else update the item
                String propertyPartNumber =
                    abOrderItem.getCatalogEntryId().toString() + "_partNumber";
                if ((orderItemsList.getString(propertyPartNumber, null)) == null) {
                    //define property names
                    String propertyOrderItemId =
                        abOrderItem.getCatalogEntryId().toString() + "_orderItemId";
                    String propertyQuantity =
                        abOrderItem.getCatalogEntryId().toString() + "_quantity";
                    String propertyStoreId =
                        abOrderItem.getCatalogEntryId().toString() + "_storeId";
                    String propertyQtyMeasure =
                        abOrderItem.getCatalogEntryId().toString() + "_qtyMeasure";

                    vCheckOrderItems.addElement(abOrderItem.getCatalogEntryId());
                    //get the unit of measure for the item
                    ItemSpecificationAccessBean itmSpcBean = new ItemSpecificationAccessBean();
                    itmSpcBean.setInitKey_itemspcId(abOrderItem.getItemSpecId());
                    itmSpcBean.refreshCopyHelper();
                    BaseItemAccessBean baseItemBean = new BaseItemAccessBean();
                    baseItemBean.setInitKey_baseItemId(itmSpcBean.getBaseItemId());
                    baseItemBean.refreshCopyHelper();
                    String qtyMeasure = baseItemBean.getQuantityMeasure();

                    orderItemsList.put(propertyOrderItemId, abOrderItem.getOrderItemId());
                    orderItemsList.put(propertyPartNumber, abOrderItem.getPartNumber());
                    orderItemsList.put(propertyQuantity, abOrderItem.getNeededQuantityInEJBType());
                    orderItemsList.put(propertyStoreId, abOrderItem.getStoreIdInEJBType());
                    orderItemsList.put(propertyQtyMeasure, qtyMeasure);

                    distinctOrderItemCount++;

                } else {
                    //update quantity if the part number matches
                    if ((orderItemsList.getString(propertyPartNumber, null))
                        .equals(abOrderItem.getPartNumber())) {
                        String propertyQuantity =
                            abOrderItem.getCatalogEntryId().toString() + "_quantity";
                        Integer newQuantity =
                            new Integer(
                                orderItemsList.getIntValue(propertyQuantity)
                                    + abOrderItem.getNeededQuantityInEJBType().intValue());
                        orderItemsList.put(propertyQuantity, newQuantity);
                    } //end if

                } // else 

            } //end while

            //to invoke Check Availability BAPI for items stored in orderItemsList propery    
            //done only for simple items, packages and bundles are not handled here

            java.util.Enumeration checkOrderItems = vCheckOrderItems.elements();

            while (checkOrderItems.hasMoreElements()) {

                String catEntryId = checkOrderItems.nextElement().toString();

                //define property names
                String propertyOrderItemId = catEntryId + "_orderItemId";
                String propertyPartNumber = catEntryId + "_partNumber";
                String propertyQuantity = catEntryId + "_quantity";
                String propertyStoreId = catEntryId + "_storeId";
                String propertyQtyMeasure = catEntryId + "_qtyMeasure";

                //get the value for check inventory
                String orderItemsId = orderItemsList.getString(propertyOrderItemId);
                String partNumber = orderItemsList.getString(propertyPartNumber);
                Integer nQuantity = orderItemsList.getInteger(propertyQuantity);
                Integer nStoreId = orderItemsList.getInteger(propertyStoreId);
                String qtyMeasure = orderItemsList.getString(propertyQtyMeasure);

                vMatAvailList =
                    checkInventoryForCatEntry(partNumber, nQuantity, nStoreId, qtyMeasure);

                getCommandContext().getRequestProperties().put(orderItemsId, vMatAvailList);

            } //end while			
        } //end if 

    } catch (RemoteException e) {
        throw new ECSystemException(
            ECMessage._ERR_REMOTE_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (CreateException e) {
        throw new ECSystemException(
            ECMessage._ERR_CREATE_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (NamingException e) {
        throw new ECSystemException(
            ECMessage._ERR_NAMING_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (FinderException e) {
        throw new ECSystemException(
            ECMessage._ERR_FINDER_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    }

    ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);

}
/**
*  Sets items to check inventory in backend system.  
*  @param	newOrderItems	
*/
public void setOrderItems(java.util.Enumeration newOrderItems) {
	ienOrderItems = newOrderItems;
}
}
