package com.ibm.commerce.sap.jca.commands;

/*
 *-------------------------------------------------------------------
 * Licensed Materials - Property of IBM
 *
 * 5724-A18
 *
 * (c) Copyright International Business Machines Corporation. yyyy, 2003
 *     All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 *-------------------------------------------------------------------
 */

//*---------------------------------------------------------------------
//* The sample contained herein is provided to you "AS IS".
//*
//* It is furnished by IBM as a simple example and has not been  
//* thoroughly tested
//* under all conditions.  IBM, therefore, cannot guarantee its 
//* reliability, serviceability or functionality.  
//*
//* This sample may include the names of individuals, companies, brands 
//* and products in order to illustrate concepts as completely as 
//* possible.  All of these names
//* are fictitious and any similarity to the names and addresses used by 
//* actual persons 
//* or business enterprises is entirely coincidental.
//*---------------------------------------------------------------------
//*


////////////////////////////////////////////////////////////////////////////////
//
// Change History:
//
// YYMMDD  WHO    Description
// --------------------------------------------------------------------
// 020527  Rajesh Danda
//
////////////////////////////////////////////////////////////////////////////////


import com.ibm.commerce.datatype.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.usermanagement.commands.*;
import com.ibm.commerce.sap.jca.util.*;
import com.ibm.commerce.util.ResourceBundleHandler;


/**
 * The new implementation class for PostAddressUpdateCmd
 * This invokes UpdateSAPAddressCmd to update the
 * address in SAP.
 */
  
public class SAPExtAddressUpdateCmdImpl extends PostAddressUpdateCmdImpl implements PostAddressUpdateCmd
{
	public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
	public final static String CLASSNAME = "com.ibm.commerce.sap.jca.commands.SAPExtAddressUpdateCmdImpl";
	protected TypedProperty requestProperties = null;
	protected TypedProperty responseProperties = null;
public TypedProperty getRequestProperties()
{
	return requestProperties;
}
public TypedProperty getResponseProperties()
{
	return responseProperties;
}
public void performExecute() throws com.ibm.commerce.exception.ECException {
    final String METHODNAME = "performExecute";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);

    //Add the business logic here
    try {
        //register only if the profile type is C or parent organization is Default Organization
        String profileType =
            requestProperties.getString(ECUserConstants.EC_USER_PROFILETYPE, "");
        String parentId =
            requestProperties.getString(ECUserConstants.EC_PARENTMEMBERID, "");
        String nickName =
            requestProperties.getString(ECUserConstants.EC_ADDR_NICKNAME, null);
        String addressId =
            requestProperties.getString(ECUserConstants.EC_ADDR_ADDRESSID, null);
        String logonID = null;

        if (addressId != null) {
            //get member id from addres access bean
            com.ibm.commerce.user.objects.AddressAccessBean addrBean =
                new com.ibm.commerce.user.objects.AddressAccessBean();
            addrBean.setInitKey_AddressId(addressId);
            addrBean.refreshCopyHelper();
            String memberID = addrBean.getMemberId();
            if (nickName == null) {
                nickName = addrBean.getNickName();
            }

            //get member type 	
            com.ibm.commerce.user.objects.MemberAccessBean mbrBean =
                new com.ibm.commerce.user.objects.MemberAccessBean();
            mbrBean.setInitKey_MemberId(memberID);
            mbrBean.refreshCopyHelper();

            if (mbrBean.getType().trim().equals("UserBean")) {
                //if member is user, get the logon id
                com.ibm.commerce.user.objects.UserRegistryAccessBean usrRegAccessBean =
                    new com.ibm.commerce.user.objects.UserRegistryAccessBean();
                usrRegAccessBean.setInitKey_UserId(memberID);
                logonID = usrRegAccessBean.getLogonId();
            } else {
                // then member type is 'O', then get the organization name
                com.ibm.commerce.user.objects.OrganizationAccessBean orgAB =
                    new com.ibm.commerce.user.objects.OrganizationAccessBean();
                orgAB.setInitKey_MemberId(memberID);
                orgAB.refreshCopyHelper();
                logonID = orgAB.getOrganizationName();
            }

            //if Parent Id is not available in request, retreive it using User Registry bean
            if (parentId.equals("")) {

                com.ibm.commerce.user.objects.UserAccessBean usersBean =
                    new com.ibm.commerce.user.objects.UserAccessBean();
                usersBean.setInitKey_MemberId(memberID);
                usersBean.refreshCopyHelper();
                parentId = usersBean.getParentMemberId();

            }

            // Here -2000, refers to the Default Organization and any
            // customer having this as the parent is assumed to be a B2C user

            if (profileType.equals(ECUserConstants.EC_USER_PROFILE_CUSTOMER)
                || (parentId.trim().equals("-2000"))) {
                //update the address in SAP only if its primary
                if (logonID.equals(nickName)) {

                    UpdateSAPAddressCmd addrEntity =
                        (UpdateSAPAddressCmd) com.ibm.commerce.command.CommandFactory.createCommand(
                            com.ibm.commerce.sap.jca.commands.UpdateSAPAddressCmd.NAME,
                            getStoreId());
                    addrEntity.setRequestProperties(requestProperties);
                    addrEntity.setResponseProperties(responseProperties);
                    addrEntity.execute();

                    addrEntity.getRequestProperties();
                    addrEntity.getResponseProperties();
                } //nickname equals logonid
            } //profile type is C or parent org is default organization
        } // if addressId is null

    } catch (javax.ejb.CreateException e) {
        throw new ECSystemException(
            ECMessage._ERR_CREATE_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    } catch (javax.ejb.FinderException e) {
        throw new ECSystemException(
            ECMessage._ERR_FINDER_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    } catch (javax.naming.NamingException e) {
        throw new ECSystemException(
            ECMessage._ERR_NAMING_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    } catch (java.rmi.RemoteException e) {
        throw new ECSystemException(
            ECMessage._ERR_REMOTE_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    }

    ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);
}
public void setRequestProperties(TypedProperty aRequestProperties)
{
	requestProperties = aRequestProperties;
}
public void setResponseProperties(TypedProperty aResponseProperties)
{
	responseProperties = aResponseProperties;
}
}
