package com.ibm.commerce.sap.jca.commands;

/*
 *-------------------------------------------------------------------
 * Licensed Materials - Property of IBM
 *
 * 5724-A18
 *
 * (c) Copyright International Business Machines Corporation. yyyy, 2003
 *     All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 *-------------------------------------------------------------------
 */

//*---------------------------------------------------------------------
//* The sample contained herein is provided to you "AS IS".
//*
//* It is furnished by IBM as a simple example and has not been  
//* thoroughly tested
//* under all conditions.  IBM, therefore, cannot guarantee its 
//* reliability, serviceability or functionality.  
//*
//* This sample may include the names of individuals, companies, brands 
//* and products in order to illustrate concepts as completely as 
//* possible.  All of these names
//* are fictitious and any similarity to the names and addresses used by 
//* actual persons 
//* or business enterprises is entirely coincidental.
//*---------------------------------------------------------------------
//*


////////////////////////////////////////////////////////////////////////////////
//
// Change History:
//
// YYMMDD  	WHO               Description
// --------------------------------------------------------------------
// 020527  	Nagaveni Sastry
//
////////////////////////////////////////////////////////////////////////////////



import javax.naming.*;
import java.util.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.exception.*;

import com.ibm.eis.sap.jca.generated.*;
import com.sap.rfc.*;
import com.sap.rfc.exception.*;
import com.ibm.connector2.sap.*;
import com.ibm.commerce.sap.jca.commands.*;
import com.ibm.commerce.util.ResourceBundleHandler;
import com.ibm.commerce.sap.jca.util.*;

/**
 * The new wrapper class for invoking OrderChange BAPI
 * to cancel the order in SAP. This is invoked by SAPOrderCreateCmdImpl
 */

public class SAPOrderCancel {
	
	private SAPConnectionSpec connSpec = new SAPConnectionSpec();
	private SAPConnectionFactory connFactory = null;
	private SAPConnection connection = null;

	OrderCancelBapi cmd = new OrderCancelBapi();

	private final static String  CLASS_NAME = "com.ibm.commerce.sap.jca.commands.SAPOrderCancel";
	private static String  istrInitFunc = "initialize";
	private static String  istrCancelOrderFunc  = "CancelOrder";
	
		
/**
 * delete a sales order in SAP
 * Calls BAPI_SALESORDER_CHANGE Bapi
 * 
 */
public void cancelOrder(String orderNumber)
    throws com.ibm.commerce.exception.ECException {

    ECTrace.entry(
        ECTraceIdentifiers.COMPONENT_EXTERN,
        CLASS_NAME,
        istrCancelOrderFunc);

    // Initialize SAP connection
    initialize();

    //  Create BAPI SALES ORDER SIMULATE
    BAPI_SALESORDER_CHANGE changeOrder = new BAPI_SALESORDER_CHANGE();

    try {
        // Set the update flag to 'D' to make it marked for delete
        Bapisdh1xStructure ordHeaderInx = new Bapisdh1xStructure();
        ordHeaderInx.setUpdateflag("D");
        changeOrder.setOrderHeaderInx(ordHeaderInx);
        changeOrder.setSalesdocument(orderNumber);
        cmd.setCeInput(changeOrder);
        cmd.execute();

        Bapiret2Table retTable = cmd.getCeOutput0().getReturn();
        String retType = retTable.getRow(0).getType();
        String retData = retTable.getRow(0).getMessage();

        if (retType.equals("E")) {
            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASS_NAME,
                istrCancelOrderFunc,
                "Order couldn't be deleted in SAP, Sales document number " + orderNumber);

             ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASS_NAME,
                istrCancelOrderFunc,
                "Action : Cancel the order in SAP, retry to delete the order using scheduler commands. If it fails, manually cancel the order in SAP");

        } else {
            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASS_NAME,
                istrCancelOrderFunc,
                "Order " + orderNumber + " deleted in SAP ");

        }

    } catch (JRfcRemoteException e) {
        throw new ECSystemException(
            ECMessageObjectsForSAP._ERR_JRFC_EXCEPTION,
            CLASS_NAME,
            istrCancelOrderFunc,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);

    } catch (Exception e) {
        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASS_NAME,
            istrCancelOrderFunc,
            e,
            true);
    }

    ECTrace.exit(
        ECTraceIdentifiers.COMPONENT_EXTERN,
        CLASS_NAME,
        istrCancelOrderFunc);

}
/**
 * Establishing the connection to SAP backend system
 */
 
 
protected void initialize() throws com.ibm.commerce.exception.ECException 
{
	ECTrace.entry(
        ECTraceIdentifiers.COMPONENT_EXTERN,
        CLASS_NAME,
        istrInitFunc);	

     try {


        // Setting properties at runtime

            Properties configProp = new Properties();
            configProp.put(
                Context.INITIAL_CONTEXT_FACTORY,               
                PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._INITIAL_CONTEXT_FACTORY));
            configProp.put(Context.PROVIDER_URL, PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._PROVIDER_URL));

            Context ctx = null;
            ctx = new javax.naming.InitialContext(configProp);
            String jndiResource = PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._JNDI_RESOURCE);
            connFactory = (com.ibm.connector2.sap.SAPConnectionFactory) ctx.lookup(jndiResource);

			connSpec.setUserName(PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._USERID));
			connSpec.setPassword(PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._PASSWD));
			 
            com.ibm.connector2.sap.SAPConnection connection =
                (com.ibm.connector2.sap.SAPConnection) connFactory.getConnection(connSpec);

            com.ibm.connector2.sap.SAPInteractionSpec ispec =
                new com.ibm.connector2.sap.SAPInteractionSpec();
            com.ibm.connector2.sap.SAPInteraction interaction =
                (com.ibm.connector2.sap.SAPInteraction) connection.createInteraction();
			connection.setTraceLevel(SAPGlobals.RAS_TRACE_INTERNAL);
			interaction.setTraceLevel(SAPGlobals.RAS_TRACE_INTERNAL);
            cmd.setCeInteractionSpec(ispec);
            cmd.setInteraction(interaction);
   

   } catch (javax.resource.ResourceException e) {
        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASS_NAME,
            istrInitFunc, 
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);    
    } catch (javax.naming.NamingException e) {
        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASS_NAME,
            istrInitFunc,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,true);     
    } catch (Exception e)	{
				
        throw new ECSystemException(ECMessage._ERR_GENERIC, CLASS_NAME, istrInitFunc, e,true);
    }

	ECTrace.exit(
        ECTraceIdentifiers.COMPONENT_EXTERN,
        CLASS_NAME,
        istrInitFunc);
		
}
}
