package com.ibm.commerce.sap.jca.commands;

/*
 *-------------------------------------------------------------------
 * Licensed Materials - Property of IBM
 *
 * 5724-A18
 *
 * (c) Copyright International Business Machines Corporation. yyyy, 2003
 *     All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 *-------------------------------------------------------------------
 */

//*---------------------------------------------------------------------
//* The sample contained herein is provided to you "AS IS".
//*
//* It is furnished by IBM as a simple example and has not been  
//* thoroughly tested
//* under all conditions.  IBM, therefore, cannot guarantee its 
//* reliability, serviceability or functionality.  
//*
//* This sample may include the names of individuals, companies, brands 
//* and products in order to illustrate concepts as completely as 
//* possible.  All of these names
//* are fictitious and any similarity to the names and addresses used by 
//* actual persons 
//* or business enterprises is entirely coincidental.
//*---------------------------------------------------------------------
//*


////////////////////////////////////////////////////////////////////////////////
//
// Change History:
//
// YYMMDD  	WHO    Description
// --------------------------------------------------------------------
// 020527  	Nagaveni Sastry
//
////////////////////////////////////////////////////////////////////////////////

/**
 *
 * This class implements the PrepareOrderCmd. The implementation
 * has been overridden for SAP.  This implementation is responsible 
 * for connecting to SAP & getting the price, inventory, schedule
 * dates, etc., from SAP & update ORDERS, ORDERITEMS tables
 * 
 */

import com.ibm.commerce.order.objects.*;
import com.ibm.commerce.order.commands.*;
import com.ibm.commerce.user.objects.*;
import com.ibm.commerce.order.utils.*;
import com.ibm.commerce.orderitems.commands.*;
import com.ibm.commerce.taxation.objects.*;
import com.ibm.commerce.util.ResourceBundleHandler;

import com.ibm.commerce.ras.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.datatype.*;

import java.util.*;
import java.rmi.*;
import javax.ejb.*;
import javax.naming.*;
import java.sql.*;
import java.math.*;

import com.ibm.eis.sap.jca.generated.*;
import com.sap.rfc.*;
import com.sap.rfc.exception.*;
import com.ibm.connector2.sap.*;
import com.ibm.commerce.sap.jca.commands.*;

import com.ibm.commerce.sap.jca.util.*;

public class SAPOrderSimulateCmdImpl
    extends PrepareOrderCmdImpl
    implements PrepareOrderCmd {
    public static final String COPYRIGHT =
        com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
    private static String CLASS_NAME =
        "com.ibm.commerce.sap.jca.commands.SAPOrderSimulateCmdImpl";

    private SAPConnectionSpec connSpec = null;
    private SAPConnectionFactory connFactory = null;
    private SAPConnection connection = null;

    OrderSimulateBapi cmd = new OrderSimulateBapi();

    public com.ibm.eis.sap.jca.generated.BapicondTable orderConditionEx;
    public com.ibm.eis.sap.jca.generated.BapisdheduTable ordScheduleEx;
    public com.ibm.eis.sap.jca.generated.BapiitemexTable OrderItemsOut;

    protected static String istrPerformFunc = "performExecute";

    TypedProperty responseProperties = null;

    private static String istrSimulateOrderFunc = "simulateOrder";
    private static String istrInitFunc = "initialize";

    public static final Long NO_CORRELATION_GROUP = new Long(-1);

    /**
     * @return com.ibm.eis.sap.jca.generated.BapicondTable
     */
    private com.ibm.eis.sap.jca.generated.BapicondTable getOrderConditionEx() {
        return orderConditionEx;
    }
    /**
     * @return com.ibm.eis.sap.jca.generated.BapiitemexTable
     */
    private com.ibm.eis.sap.jca.generated.BapiitemexTable getOrderItemsOut() {
        return OrderItemsOut;
    }
    /**
     * Gets ids of the orders to be prepared.
     */
    public java.util.Vector getOrders() {
        return super.getOrders();
    }
    /**
     * @return com.ibm.eis.sap.jca.generated.BapisdheduTable
     */
    private com.ibm.eis.sap.jca.generated.BapisdheduTable getOrdScheduleEx() {
        return ordScheduleEx;
    }
/**
 * Establishing the connection to SAP backend system
 */
 

   protected void initialize() throws com.ibm.commerce.exception.ECException {
        ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASS_NAME, istrInitFunc);
        
        try {


            // Setting properties at runtime


        
                Properties configProp = new Properties();

                configProp.put(
                    Context.INITIAL_CONTEXT_FACTORY,
                    PropertiesKeysForSAP.sapResourceBundle.getString(
                        PropertiesKeysForSAP._INITIAL_CONTEXT_FACTORY));
                configProp.put(
                    Context.PROVIDER_URL,
                    PropertiesKeysForSAP.sapResourceBundle.getString(
                        PropertiesKeysForSAP._PROVIDER_URL));

                Context ctx = null;
                ctx = new javax.naming.InitialContext(configProp);

                String jndiResource =
                    PropertiesKeysForSAP.sapResourceBundle.getString(
                        PropertiesKeysForSAP._JNDI_RESOURCE);

                com.ibm.connector2.sap.SAPConnectionFactory scf =
                    (com.ibm.connector2.sap.SAPConnectionFactory) ctx.lookup(jndiResource);

                com.ibm.connector2.sap.SAPConnectionSpec scs =
                    new com.ibm.connector2.sap.SAPConnectionSpec();

                scs.setUserName(
                    ResourceBundleHandler.getStringByKey(
                        PropertiesKeysForSAP._USERID,
                        PropertiesKeysForSAP.sapResourceBundle));
                scs.setPassword(
                    ResourceBundleHandler.getStringByKey(
                        PropertiesKeysForSAP._PASSWD,
                        PropertiesKeysForSAP.sapResourceBundle));

                com.ibm.connector2.sap.SAPConnection connection =
                    (com.ibm.connector2.sap.SAPConnection) scf.getConnection(scs);
                com.ibm.connector2.sap.SAPInteractionSpec ispec =
                    new com.ibm.connector2.sap.SAPInteractionSpec();
                com.ibm.connector2.sap.SAPInteraction interaction =
                    (com.ibm.connector2.sap.SAPInteraction) connection.createInteraction();

                cmd.setCeInteractionSpec(ispec);
                cmd.setInteraction(interaction);

		        
        } catch (javax.resource.ResourceException e) {
        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASS_NAME,
            istrInitFunc, 
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);    
    } catch (javax.naming.NamingException e) {
        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASS_NAME,
            istrInitFunc,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,true);   
    }  catch (Exception e) {   
	    
         
            throw new ECSystemException(
                ECMessage._ERR_GENERIC,
                CLASS_NAME,
                istrInitFunc,
                e,true);
        }

        ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASS_NAME, istrInitFunc);

    }
/**
 * Contains the actual business logic of the command.
 */

public void performExecute() throws ECException {
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASS_NAME, istrPerformFunc);

    try {
        if (getOrders().isEmpty())
            return;

        // prepare the order, getOrders() returns the current order for the user,
        // SAPToolTech store maintains one pending order per customer.

        for (Enumeration enOrders = getOrders().elements();
            enOrders.hasMoreElements();
            ) {
            OrderAccessBean abOrder = (OrderAccessBean) enOrders.nextElement();

            Long nOrderCustomer = abOrder.getMemberIdInEJBType();
            String sOrderId = abOrder.getOrderId();

            // call simulate order 
            simulateOrder(sOrderId);

            //results from SAP must be for the same number of elements as the number of elements in the order
            if (getOrderItemsOut().getRowCount() == abOrder.getOrderItems().length) {

                int orderItemOutIndex = 0;

                while (orderItemOutIndex < getOrderItemsOut().getRowCount()) {

                    int schLineIndex = 0;

                    double shippingCostPerUnit = 0.0;
                    double taxAmountPerUnit = 0.0;
                    double shippingItemTotal = 0.0;
                    double taxItemTotal = 0.0;
                    double shippingTaxTotal = 0.0;
                    double shippingTaxPerUnit = 0.0;
                    double priceTotal = 0.0;
                    BigDecimal unitPrice = new BigDecimal(0);

                    //get the order item from WC 
                    OrderItemAccessBean oIABCurrentItem = new OrderItemAccessBean();
                    oIABCurrentItem.setInitKey_orderItemId(
                        getOrderItemsOut().getRow(orderItemOutIndex).getItmNumberString());
                    oIABCurrentItem.refreshCopyHelper();

                    //get the shipping info and tax info from OrderItemsOut
                    // subtotal1 : price
                    // subtotal4 : shipping
                    // subtotal5 : tax
                    // subtotal6 : shippingtax

                    if (getOrderItemsOut().getRow(orderItemOutIndex).getSubtotal1().doubleValue()
                        > 0) {
                        priceTotal =
                            getOrderItemsOut().getRow(orderItemOutIndex).getSubtotal1().doubleValue();
                        unitPrice =
                            (new BigDecimal(priceTotal
                                / (getOrderItemsOut().getRow(orderItemOutIndex).getReqQty().doubleValue())))
                                .movePointLeft(2)
                                .setScale(2, BigDecimal.ROUND_UP);
                    }

                    if (getOrderItemsOut().getRow(orderItemOutIndex).getSubtotal4().doubleValue()
                        > 0) {
                        shippingItemTotal =
                            getOrderItemsOut().getRow(orderItemOutIndex).getSubtotal4().doubleValue();
                        shippingCostPerUnit =
                            shippingItemTotal
                                / (getOrderItemsOut().getRow(orderItemOutIndex).getReqQty().doubleValue());

                    }
                    if (getOrderItemsOut().getRow(orderItemOutIndex).getSubtotal5().doubleValue()
                        > 0) {
                        taxItemTotal =
                            getOrderItemsOut().getRow(orderItemOutIndex).getSubtotal5().doubleValue();
                        taxAmountPerUnit =
                            taxItemTotal
                                / (getOrderItemsOut().getRow(orderItemOutIndex).getReqQty().doubleValue());
                    }
                    if (getOrderItemsOut().getRow(orderItemOutIndex).getSubtotal6().doubleValue()
                        > 0) {
                        shippingTaxTotal =
                            getOrderItemsOut().getRow(orderItemOutIndex).getSubtotal6().doubleValue();
                        shippingTaxPerUnit =
                            shippingTaxTotal
                                / (getOrderItemsOut().getRow(orderItemOutIndex).getReqQty().doubleValue());
                    }


                    schLineIndex = 0;
                    int firstValidSchedule = 1;
                    while (schLineIndex < getOrdScheduleEx().getRowCount()) {
                        //if conidition to check whether the schedule ItemNumber matches with its parent Item Number
                        if (getOrdScheduleEx()
                            .getRow(schLineIndex)
                            .getItmNumberString()
                            .equalsIgnoreCase(
                                getOrderItemsOut().getRow(orderItemOutIndex).getItmNumberString())) {
                            //condition to ignore schedule lines with quantity 0
                            if (!(getOrdScheduleEx().getRow(schLineIndex).getConfirQty().doubleValue()
                                == (new Double("0").doubleValue()))) {
                                //update the existing item, if its first valid schedule
                                if (firstValidSchedule == 1) {
                                    oIABCurrentItem.setLastUpdate(new Timestamp(System.currentTimeMillis()));

                                    // set correlation group id if it is empty
                                    if (oIABCurrentItem.getCorrelationGroupInEJBType() == null
                                        || oIABCurrentItem.getCorrelationGroupInEJBType() == NO_CORRELATION_GROUP) {
                                        oIABCurrentItem.setCorrelationGroup(oIABCurrentItem.getOrderItemIdInEJBType());
                                    }

                                    //make sure quantity is converted from string to double or so
                                    //also while merging it should update qty and needed qty

                                    //code for rounding to the nearest whole number
                                    int qtyValue = 0;
                                    Double originalValue =
                                        new Double(
                                            getOrdScheduleEx().getRow(schLineIndex).getConfirQty().doubleValue());
                                    Double truncValue =
                                        new Double(
                                            String.valueOf(
                                                getOrdScheduleEx().getRow(schLineIndex).getConfirQty().intValue()));
                                    Double diffValue = new Double("0.50");

                                    if ((originalValue.doubleValue() - truncValue.doubleValue())
                                        >= diffValue.doubleValue())
                                        qtyValue = 1;

                                    oIABCurrentItem.setQuantity(
                                        String.valueOf(
                                            getOrdScheduleEx().getRow(schLineIndex).getConfirQty().intValue() + qtyValue));
                                    oIABCurrentItem.setNeededQuantity(
                                        new Integer(
                                            getOrdScheduleEx().getRow(schLineIndex).getConfirQty().intValue() + qtyValue));
                                    oIABCurrentItem.setAllocatedQuantity(
                                        new Integer(
                                            getOrdScheduleEx().getRow(schLineIndex).getConfirQty().intValue() + qtyValue));

                                    // oIABCurrentItem.setField1(getOrdScheduleEx().getRow(schLineIndex).getSchedLineString());
                                    oIABCurrentItem.setField1(new Integer(1));
                                    // changed to put the same value for the items that have been splitted
                                    oIABCurrentItem.setPromisedAvailableTime(
                                        new Timestamp(getOrdScheduleEx().getRow(schLineIndex).getMsDate().getTime()));
                                    oIABCurrentItem.setEstimatedAvailableTime(
                                        new Timestamp(getOrdScheduleEx().getRow(schLineIndex).getMsDate().getTime()));

                                    // get the shipping, tax, shipping tax cost for one unit

                                    BigDecimal shippingCharge = new BigDecimal("0");
                                    double shippingChargeForConfirQty =
                                        shippingCostPerUnit
                                            * getOrdScheduleEx().getRow(schLineIndex).getConfirQty().doubleValue();
                                    if (shippingChargeForConfirQty > 0.01)
                                        shippingCharge =
                                            (new BigDecimal(shippingChargeForConfirQty)).movePointLeft(2).setScale(
                                                2,
                                                BigDecimal.ROUND_UP);

                                    BigDecimal taxAmount = new BigDecimal("0");
                                    double taxForConfirQty =
                                        taxAmountPerUnit
                                            * getOrdScheduleEx().getRow(schLineIndex).getConfirQty().doubleValue();
                                    if (taxForConfirQty > 0.01)
                                        taxAmount =
                                            (new BigDecimal(taxForConfirQty)).movePointLeft(2).setScale(
                                                2,
                                                BigDecimal.ROUND_UP);

                                    BigDecimal shippingTax = new BigDecimal("0");
                                    double shipTaxForConfirQty =
                                        shippingTaxPerUnit
                                            * getOrdScheduleEx().getRow(schLineIndex).getConfirQty().doubleValue();
                                    if (shipTaxForConfirQty > 0.01)
                                        shippingTax =
                                            (new BigDecimal(shipTaxForConfirQty)).movePointLeft(2).setScale(
                                                2,
                                                BigDecimal.ROUND_UP);

                                    oIABCurrentItem.setPrice(unitPrice);
                                    oIABCurrentItem.setPrepareFlags(
                                        new Integer(OrderConstants.PREPAREFLAGS_PRICE_OVERRIDE));


                                    oIABCurrentItem.setShippingCharge(shippingCharge.toString());
                                    oIABCurrentItem.setTaxAmount(taxAmount.toString());
                                    oIABCurrentItem.setShippingTaxAmount(shippingTax.toString());

                                    oIABCurrentItem.commitCopyHelper();

                                    firstValidSchedule = 0;

                                } //if firstValidateSchedule
                                //if more than one schedule line with qty>0, insert another order item					        
                                else
                                    if (firstValidSchedule == 0) {
                                        ECTrace.trace(
                                            ECTraceIdentifiers.COMPONENT_EXTERN,
                                            CLASS_NAME,
                                            istrPerformFunc,
                                            "Order item has been split according to the schedule lines === schedule line nubmer : "
                                                + getOrdScheduleEx().getRow(schLineIndex).getSchedLineString());

                                        TypedProperty requestProperties = new TypedProperty();
                                        OrderItemAddCmd copyItem =
                                            (OrderItemAddCmd) CommandFactory.createCommand(
                                                OrderItemAddCmd.NAME,
                                                getCommandContext().getStore().getStoreEntityIdInEJBType());
                                        requestProperties.put("URL", "/");
                                        requestProperties.put(ECConstants.EC_STORE_ID, oIABCurrentItem.getStoreId());
                                        requestProperties.put(OrderConstants.EC_ORDER_ID, oIABCurrentItem.getOrderId());
                                        requestProperties.put(
                                            OrderConstants.EC_CATENTRY_ID,
                                            oIABCurrentItem.getCatalogEntryId());
                                        requestProperties.put(
                                            OrderConstants.EC_QUANTITY,
                                            getOrdScheduleEx().getRow(schLineIndex).getConfirQtyString());
                                        requestProperties.put(
                                            OrderConstants.EC_SHIPMODE_ID,
                                            oIABCurrentItem.getShippingModeId());
                                        requestProperties.put(
                                            OrderConstants.EC_CURRENCY,
                                            oIABCurrentItem.getCurrency());
                                        requestProperties.put(OrderConstants.EC_PRICE, unitPrice);
                                        requestProperties.put(
                                            OrderConstants.EC_CORRELATTION_GROUP,
                                            oIABCurrentItem.getCorrelationGroup());

                                        copyItem.setRequestProperties(requestProperties);
                                        copyItem.setCommandContext(getCommandContext());
                                        copyItem.execute();

                                        OrderItemAccessBean oIABNewItem = new OrderItemAccessBean();
                                        //ideally copyitem should contain only one orderitem id in the request. need to clarify this
                                        oIABNewItem.setInitKey_orderItemId(copyItem.getOrderItemIds()[0]);
                                        oIABNewItem.refreshCopyHelper();

                                        //code for truncation
                                        int qtyValue = 0;
                                        Double originalValue =
                                            new Double(
                                                getOrdScheduleEx().getRow(schLineIndex).getConfirQty().doubleValue());
                                        Double truncValue =
                                            new Double(
                                                String.valueOf(
                                                    getOrdScheduleEx().getRow(schLineIndex).getConfirQty().intValue()));
                                        Double diffValue = new Double("0.50");

                                        if ((originalValue.doubleValue() - truncValue.doubleValue())
                                            >= diffValue.doubleValue())
                                            qtyValue = 1;

                                        oIABNewItem.setQuantity(
                                            String.valueOf(
                                                getOrdScheduleEx().getRow(schLineIndex).getConfirQty().intValue() + qtyValue));
                                        oIABNewItem.setNeededQuantity(
                                            new Integer(
                                                getOrdScheduleEx().getRow(schLineIndex).getConfirQty().intValue() + qtyValue));
                                        oIABNewItem.setAllocatedQuantity(
                                            new Integer(
                                                getOrdScheduleEx().getRow(schLineIndex).getConfirQty().intValue() + qtyValue));

                                        // oIABNewItem.setField1(getOrdScheduleEx().getRow(schLineIndex).getSchedLineString());
                                        oIABNewItem.setField1(new Integer(1));
                                        // changed to put the same value for the items that have been splitted
                                        oIABNewItem.setPromisedAvailableTime(
                                            new Timestamp(getOrdScheduleEx().getRow(schLineIndex).getMsDate().getTime()));
                                        oIABNewItem.setEstimatedAvailableTime(
                                            new Timestamp(getOrdScheduleEx().getRow(schLineIndex).getMsDate().getTime()));

                                        //setting price, tax, shipping information using COND table
                                        //make sure tax and shipping calculation are for not associate with the quantity

                                        BigDecimal shippingCharge = new BigDecimal("0");
                                        double shippingChargeForConfirQty =
                                            shippingCostPerUnit
                                                * getOrdScheduleEx().getRow(schLineIndex).getConfirQty().doubleValue();
                                        if (shippingChargeForConfirQty > 0.01)
                                            shippingCharge =
                                                (new BigDecimal(shippingChargeForConfirQty)).movePointLeft(2).setScale(
                                                    2,
                                                    BigDecimal.ROUND_UP);

                                        BigDecimal taxAmount = new BigDecimal("0");
                                        double taxForConfirQty =
                                            taxAmountPerUnit
                                                * getOrdScheduleEx().getRow(schLineIndex).getConfirQty().doubleValue();
                                        if (taxForConfirQty > 0.01)
                                            taxAmount =
                                                (new BigDecimal(taxForConfirQty)).movePointLeft(2).setScale(
                                                    2,
                                                    BigDecimal.ROUND_UP);

                                        BigDecimal shippingTax = new BigDecimal("0");
                                        double shipTaxForConfirQty =
                                            shippingTaxPerUnit
                                                * getOrdScheduleEx().getRow(schLineIndex).getConfirQty().doubleValue();
                                        if (shipTaxForConfirQty > 0.01)
                                            shippingTax =
                                                (new BigDecimal(shipTaxForConfirQty)).movePointLeft(2).setScale(
                                                    2,
                                                    BigDecimal.ROUND_UP);

                                        oIABNewItem.setPrice(unitPrice);
                                        oIABNewItem.setPrepareFlags(
                                            new Integer(OrderConstants.PREPAREFLAGS_PRICE_OVERRIDE));
                                        oIABNewItem.setShippingCharge(shippingCharge.toString());
                                        oIABNewItem.setTaxAmount(taxAmount.toString());
                                        oIABNewItem.setShippingTaxAmount(shippingTax.toString());

                                        oIABNewItem.commitCopyHelper();


                                    } //else if for creating and updating the new item
                            } //if quantity is 0
                        } //if itemnumber matches with orderitemid

                        schLineIndex++; //increment the while loop counter
                    } // end of while - schedule lines             

                    orderItemOutIndex++; //increment the while loop counter
                } //end of while - OrderItemOut

            } //end if to compare the WCorderitemscount and SAPorderitemscount

        } //end of while - order array

        super.performExecute();

        // set the shipping, tax, shiptax totals for the order

        for (Enumeration enOrders = getOrders().elements();            enOrders.hasMoreElements();
            ) {

            double orderShippingTotal = 0.0;
            double orderTaxTotal = 0.0;
            double orderShiptaxTotal = 0.0;

            OrderAccessBean abOrder = (OrderAccessBean) enOrders.nextElement();

            OrderItemAccessBean[] items = abOrder.getOrderItems();
            for (int itemIndex = 0; itemIndex < items.length; itemIndex++) {

                orderShippingTotal
                    += items[itemIndex].getShippingChargeInEJBType().doubleValue();
                orderTaxTotal += items[itemIndex].getTaxAmountInEJBType().doubleValue();
                orderShiptaxTotal
                    += items[itemIndex].getShippingTaxAmountInEJBType().doubleValue();

            }

            BigDecimal TotalOrderShipping =
                (new BigDecimal(orderShippingTotal)).setScale(2, BigDecimal.ROUND_UP);
            BigDecimal TotalOrderTax =
                (new BigDecimal(orderTaxTotal)).setScale(2, BigDecimal.ROUND_UP);
            BigDecimal TotalOrderShiptax =
                (new BigDecimal(orderShiptaxTotal)).setScale(2, BigDecimal.ROUND_UP);


            abOrder.setTotalShippingCharge(TotalOrderShipping.toString());
            abOrder.setTotalTax(TotalOrderTax.toString());
            abOrder.setTotalShippingTax(TotalOrderShiptax.toString());
            abOrder.commitCopyHelper();

        }
    } catch (JRfcRemoteException e) {
        throw new ECSystemException(
            ECMessageObjectsForSAP._ERR_JRFC_EXCEPTION,
            CLASS_NAME,
            istrPerformFunc,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);

    } catch (RemoteException e) {
        throw new ECSystemException(
            ECMessage._ERR_REMOTE_EXCEPTION,
            CLASS_NAME,
            istrPerformFunc,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (CreateException e) {
        throw new ECSystemException(
            ECMessage._ERR_CREATE_EXCEPTION,
            CLASS_NAME,
            istrPerformFunc,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (NamingException e) {
        throw new ECSystemException(
            ECMessage._ERR_NAMING_EXCEPTION,
            CLASS_NAME,
            istrPerformFunc,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (FinderException e) {
        throw new ECSystemException(
            ECMessage._ERR_FINDER_EXCEPTION,
            CLASS_NAME,
            istrPerformFunc,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } 

    ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASS_NAME, istrPerformFunc);
}
    /**
     * @param newOrderConditionEx com.ibm.eis.sap.jca.generated.BapicondTable
     */
    private void setOrderConditionEx(
        com.ibm.eis.sap.jca.generated.BapicondTable newOrderConditionEx) {
        orderConditionEx = newOrderConditionEx;
    }
    /**
     * @param newOrderItemsOut com.ibm.eis.sap.jca.generated.BapiitemexTable
     */
    private void setOrderItemsOut(
        com.ibm.eis.sap.jca.generated.BapiitemexTable newOrderItemsOut) {
        OrderItemsOut = newOrderItemsOut;
    }
    /**
     * @param newOrdSchedule com.ibm.eis.sap.jca.generated.BapisdheduTable
     */
    private void setOrdScheduleEx(
        com.ibm.eis.sap.jca.generated.BapisdheduTable newOrdSchedule) {
        ordScheduleEx = newOrdSchedule;
    }
/**
 * Calls BAPI_SALESORDER_SIMULATE Bapi to 
 * calculate order cost and availability dates
 */

private void simulateOrder(String orderNumber)
    throws com.ibm.commerce.exception.ECException {

    ECTrace.entry(
        ECTraceIdentifiers.COMPONENT_EXTERN,
        CLASS_NAME,
        istrSimulateOrderFunc);

    //  Create BAPI SALES ORDER SIMULATE
    BAPI_SALESORDER_SIMULATE ordSim = new BAPI_SALESORDER_SIMULATE();

    try {
        // initialize SAP connection 
        initialize();

        String doctype =
            ResourceBundleHandler.getStringByKey(
                PropertiesKeysForSAP._DOC_TYPE,
                PropertiesKeysForSAP.sapResourceBundle);
        String salesorg =
            ResourceBundleHandler.getStringByKey(
                PropertiesKeysForSAP._SALES_ORGANIZATION,
                PropertiesKeysForSAP.sapResourceBundle);
        String distrchan =
            ResourceBundleHandler.getStringByKey(
                PropertiesKeysForSAP._DISTRIBUTION_CHANNEL,
                PropertiesKeysForSAP.sapResourceBundle);
        String division =
            ResourceBundleHandler.getStringByKey(
                PropertiesKeysForSAP._DIVISION,
                PropertiesKeysForSAP.sapResourceBundle);

        OrderAccessBean orderBean = new OrderAccessBean();
        orderBean.setInitKey_orderId(orderNumber);
        orderBean.refreshCopyHelper();

        // get the shipping mode id from the request	
        String requestShipModeId =
            getCommandContext().getRequestProperties().getString("shippingMode", null);
        if ((requestShipModeId == null) || (requestShipModeId.trim().equals(""))) {

            // If not available in the request, then take it from the orderitems table
            // Take the shippingmode for the order from the first item as the default.

            OrderItemAccessBean[] orderItems = orderBean.getOrderItems();
            requestShipModeId = orderItems[0].getShippingModeId();

        }
        // assign the carrier as incoterms1 and code as incoterms2
        com.ibm.commerce.fulfillment.objects.ShippingModeAccessBean shipMode =
            new com.ibm.commerce.fulfillment.objects.ShippingModeAccessBean();
        shipMode.setInitKey_shippingModeId(requestShipModeId);
        shipMode.refreshCopyHelper();
        String incot1 = shipMode.getCarrier().trim();
        String incot2 = shipMode.getCode().trim();

        String plant =
            ResourceBundleHandler.getStringByKey(
                PropertiesKeysForSAP._PLANT,
                PropertiesKeysForSAP.sapResourceBundle);

        String partRoleSP =
            ResourceBundleHandler.getStringByKey(
                PropertiesKeysForSAP._PART_ROLE_SP,
                PropertiesKeysForSAP.sapResourceBundle);

        String partRoleSH =
            ResourceBundleHandler.getStringByKey(
                PropertiesKeysForSAP._PART_ROLE_SH,
                PropertiesKeysForSAP.sapResourceBundle);

        String partRoleBP =
            ResourceBundleHandler.getStringByKey(
                PropertiesKeysForSAP._PART_ROLE_BP,
                PropertiesKeysForSAP.sapResourceBundle);

        java.util.Date reqDate = new java.util.Date();
        java.util.Date purchDate = new java.util.Date();
        java.util.Date itemReqDate = new java.util.Date();

        String curr = getCommandContext().getCurrency();

        // set OrderHeaderIn - It is same for all orders

        BapisdheadStructure ordHeadIn = new BapisdheadStructure();

        ordHeadIn.setDocType(doctype);
        ordHeadIn.setSalesOrg(salesorg);
        ordHeadIn.setDistrChan(distrchan);
        ordHeadIn.setDivision(division);
        ordHeadIn.setReqDateH(reqDate);
        ordHeadIn.setPurchNo(orderNumber);
        ordHeadIn.setPurchDate(purchDate);
        ordHeadIn.setIncoterms1(incot1);
        ordHeadIn.setIncoterms2(incot2);
        ordHeadIn.setCurrency(curr);
        ordSim.setOrderHeaderIn(ordHeadIn);

        BapiiteminTable itemsInTable = new BapiiteminTable();
        BapipartnrTable partTable = new BapipartnrTable();

        String partnerNumber = null;

        // Here -2000, refers to the Default Organization and any
        // customer having this as the parent is assumed to be a B2C user

        if (getUser()
            .getProfileType()
            .equals(com.ibm.commerce.user.helpers.ECUserConstants.EC_USER_PROFILE_CUSTOMER)
            || (getUser().getParentMemberId().equals("-2000"))) {
            partnerNumber = getUser().getUserField1();
        } else {
            OrganizationAccessBean orgAB = new OrganizationAccessBean();
            orgAB.setInitKey_MemberId(getUser().getParentMemberId());
            orgAB.refreshCopyHelper();
            partnerNumber = orgAB.getOrgEntityField1();
        }


        OrderAccessBean abOrder = new OrderAccessBean();
        abOrder.setInitKey_orderId(orderNumber);
        abOrder.refreshCopyHelper();

        OrderItemAccessBean[] abOIArray = abOrder.getOrderItems();

        // simulate for all orderitems in the order
        for (int a = 0; a < abOIArray.length; a++) {

            //set OrderItemsIn
            BapiiteminTableRow itemsInTableRow = itemsInTable.createEmptyRow();
            itemsInTableRow.setItmNumberString(abOIArray[a].getOrderItemId());
            itemsInTableRow.setPoItmNo(abOIArray[a].getOrderItemId());

            // set the partnumber as the material - synchronise the data in WCS to be same as SAP 
            itemsInTableRow.setMaterial(abOIArray[a].getPartNumber());
            itemsInTableRow.setPlant(plant);
            itemsInTableRow.setReqQtyString(
                abOIArray[a].getNeededQuantityInEJBType().intValue() + "000");
            // Refer to BAPI dcoumentation for appending "000"
            itemsInTableRow.setReqDate(itemReqDate);

            String itemShipModeId = abOIArray[a].getShippingModeId();
            com.ibm.commerce.fulfillment.objects.ShippingModeAccessBean shippingBean =
                new com.ibm.commerce.fulfillment.objects.ShippingModeAccessBean();

            if ((itemShipModeId != null) && !(itemShipModeId.equals(""))) {

                shippingBean.setInitKey_shippingModeId(itemShipModeId);
                shippingBean.refreshCopyHelper();
                itemsInTableRow.setIncoterms1(shippingBean.getCarrier());
                itemsInTableRow.setIncoterms2(shippingBean.getCode());
            } else {
                itemsInTableRow.setIncoterms1(incot1);
                itemsInTableRow.setIncoterms2(incot2);
            }

            itemsInTable.appendRow(itemsInTableRow);

            //partner function at orderitem level
            //set OrderPartners for ship-to "SH" if not null
            if ((partRoleSH != null) && !(partRoleSH.trim().equals(""))) {
                BapipartnrTableRow partTableRow = partTable.createEmptyRow();
                partTableRow.setPartnRole(partRoleSH);

                partTableRow.setPartnNumb(partnerNumber);
                partTableRow.setItmNumberString(abOIArray[a].getOrderItemId());

                AddressAccessBean shipAddrBean = new AddressAccessBean();
                shipAddrBean.setInitKey_AddressId(abOIArray[a].getAddressId());
                shipAddrBean.refreshCopyHelper();

                StringBuffer strBuffer = new StringBuffer();
                if ((shipAddrBean.getAddress1() != null)
                    && !(shipAddrBean.getAddress1().trim().equals("")))
                    strBuffer.append(shipAddrBean.getAddress1().trim());
                if ((shipAddrBean.getAddress2() != null)
                    && !(shipAddrBean.getAddress2().trim().equals("")))
                    strBuffer.append(shipAddrBean.getAddress2().trim());
                if ((shipAddrBean.getAddress3() != null)
                    && !(shipAddrBean.getAddress3().trim().equals("")))
                    strBuffer.append(shipAddrBean.getAddress3().trim());

                if ((strBuffer.length() > 0) && (strBuffer != null)) {
                    if (strBuffer.length() > 35) {
                        partTableRow.setStreet(strBuffer.substring(0, 34));
                    } else {
                        partTableRow.setStreet(strBuffer.toString());
                    }
                }

                if ((shipAddrBean.getCity() != null)
                    && !(shipAddrBean.getCity().trim().equals("")))
                    partTableRow.setCity(shipAddrBean.getCity().trim());
                if ((shipAddrBean.getCountry() != null)
                    && !(shipAddrBean.getCountry().trim().equals("")))
                    partTableRow.setCountry(shipAddrBean.getCountry().trim());

                //to get SAP statecode for a given wc state(s) 
                String stateAbbr = shipAddrBean.getState().trim();
                if ((stateAbbr != null) && !(stateAbbr.equals(""))) {

                    StateCodeAccessBean stateCode = new StateCodeAccessBean();
                    stateCode.setInitKey_strStateAbbr(stateAbbr);
                    stateCode.setInitKey_strProductVendor(
                        PropertiesKeysForSAP.sapResourceBundle.getString(
                            PropertiesKeysForSAP._PRODUCT_VENDOR));
                    stateCode.refreshCopyHelper();

                    partTableRow.setRegion(stateCode.getStateCode());
                }
                if ((shipAddrBean.getZipCode() != null)
                    && !(shipAddrBean.getZipCode().trim().equals("")))
                    partTableRow.setPostlCode(shipAddrBean.getZipCode());
                if ((shipAddrBean.getLastName() != null)
                    && !(shipAddrBean.getLastName().trim().equals("")))
                    partTableRow.setName(shipAddrBean.getLastName());

                partTable.appendRow(partTableRow);

            } // (if  partRoleSH != null)			

        } // for order items

        //partner functions at order level

        //set OrderPartners for bill-to "BP"
        if ((partRoleBP != null) && !(partRoleBP.trim().equals(""))) {

            BapipartnrTableRow partTableRow = partTable.createEmptyRow();
            partTableRow.setPartnRole(partRoleBP);
            partTableRow.setPartnNumb(partnerNumber);

            AddressAccessBean billAddrBean = new AddressAccessBean();
            billAddrBean.setInitKey_AddressId(orderBean.getAddressId());
            billAddrBean.refreshCopyHelper();

            StringBuffer strBuffer = new StringBuffer();
            if ((billAddrBean.getAddress1() != null)
                && !(billAddrBean.getAddress1().trim().equals("")))
                strBuffer.append(billAddrBean.getAddress1().trim());
            if ((billAddrBean.getAddress2() != null)
                && !(billAddrBean.getAddress2().trim().equals("")))
                strBuffer.append(billAddrBean.getAddress2().trim());
            if ((billAddrBean.getAddress3() != null)
                && !(billAddrBean.getAddress3().trim().equals("")))
                strBuffer.append(billAddrBean.getAddress3().trim());

            if ((strBuffer.length() > 0) && (strBuffer != null)) {
                if (strBuffer.length() > 35) {
                    partTableRow.setStreet(strBuffer.substring(0, 34));
                } else {
                    partTableRow.setStreet(strBuffer.toString());
                }
            }

            if ((billAddrBean.getCity() != null)
                && !(billAddrBean.getCity().trim().equals("")))
                partTableRow.setCity(billAddrBean.getCity().trim());
            if ((billAddrBean.getCountry() != null)
                && !(billAddrBean.getCountry().trim().equals("")))
                partTableRow.setCountry(billAddrBean.getCountry().trim());

            //to get SAP statecode for a given wc state(s) 
            String stateAbbr = billAddrBean.getState().trim();
            if ((stateAbbr != null) && !(stateAbbr.equals(""))) {

                StateCodeAccessBean stateCode = new StateCodeAccessBean();
                stateCode.setInitKey_strStateAbbr(stateAbbr);
                stateCode.setInitKey_strProductVendor(
                    PropertiesKeysForSAP.sapResourceBundle.getString(
                        PropertiesKeysForSAP._PRODUCT_VENDOR));
                stateCode.refreshCopyHelper();

                partTableRow.setRegion(stateCode.getStateCode());
            }
            if ((billAddrBean.getZipCode() != null)
                && !(billAddrBean.getZipCode().trim().equals("")))
                partTableRow.setPostlCode(billAddrBean.getZipCode());
            if ((billAddrBean.getLastName() != null)
                && !(billAddrBean.getLastName().trim().equals("")))
                partTableRow.setName(billAddrBean.getLastName());

            partTable.appendRow(partTableRow);
        } // if (partRoleBP != null)

        //set OrderPartners for sold-to "SP"
        if ((partRoleSP != null) && !(partRoleSP.trim().equals(""))) {

            BapipartnrTableRow partTableRow = partTable.createEmptyRow();
            partTableRow.setPartnRole(partRoleSP);

            partTableRow.setPartnNumb(partnerNumber);

            partTable.appendRow(partTableRow);
        } // if (partRoleSP != null)
        // end of address stuff


        ordSim.setOrderHeaderIn(ordHeadIn);
        ordSim.setOrderItemsIn(itemsInTable);
        ordSim.setOrderPartners(partTable);
        ordSim.setConvertParvwAuart("X"); // to convert the language

        cmd.setCeInput(ordSim);

        // execute the command
        cmd.execute();



        BapireturnStructure retStruct = cmd.getCeOutput0().getReturn();

        String retType = retStruct.getType();
        String retData = retStruct.getMessage();

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            istrSimulateOrderFunc,
            "**SAP Return Type** " + retType);
        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASS_NAME,
            istrSimulateOrderFunc,
            "**SAP Return Data** " + retData);

        if (retType.equals("E")) {
            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASS_NAME,
                istrSimulateOrderFunc,
                "Failed to do Order Simulate in SAP");

            throw new ECApplicationException(
                ECMessageObjectsForSAP._ERR_SIMULATE_ORDER,
                CLASS_NAME,
                istrSimulateOrderFunc,
                "GenericApplicationError",
                true);

        } else {

            setOrdScheduleEx(cmd.getCeOutput0().getOrderScheduleEx());
            setOrderItemsOut(cmd.getCeOutput0().getOrderItemsOut());
            setOrderConditionEx(cmd.getCeOutput0().getOrderConditionEx());

            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASS_NAME,
                istrSimulateOrderFunc,
                " Order Simulate is successful ");

        }

    } catch (JRfcRemoteException e) {
        throw new ECSystemException(
            ECMessageObjectsForSAP._ERR_JRFC_EXCEPTION,
            CLASS_NAME,
            istrSimulateOrderFunc,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    } catch (javax.ejb.CreateException e) {
        throw new ECSystemException(
            ECMessage._ERR_CREATE_EXCEPTION,
            CLASS_NAME,
            istrSimulateOrderFunc,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    } catch (javax.ejb.FinderException e) {
        throw new ECSystemException(
            ECMessage._ERR_FINDER_EXCEPTION,
            CLASS_NAME,
            istrSimulateOrderFunc,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    } catch (javax.naming.NamingException e) {
        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASS_NAME,
            istrSimulateOrderFunc,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    } catch (java.rmi.RemoteException e) {
        throw new ECSystemException(
            ECMessage._ERR_REMOTE_EXCEPTION,
            CLASS_NAME,
            istrSimulateOrderFunc,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);
    } 

    ECTrace.exit(
        ECTraceIdentifiers.COMPONENT_EXTERN,
        CLASS_NAME,
        istrSimulateOrderFunc);

}

}
