package com.ibm.commerce.sap.jca.commands;

/*
 *-------------------------------------------------------------------
 * Licensed Materials - Property of IBM
 *
 * 5724-A18
 *
 * (c) Copyright International Business Machines Corporation. yyyy, 2003
 *     All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 *-------------------------------------------------------------------
 */

//*---------------------------------------------------------------------
//* The sample contained herein is provided to you "AS IS".
//*
//* It is furnished by IBM as a simple example and has not been  
//* thoroughly tested
//* under all conditions.  IBM, therefore, cannot guarantee its 
//* reliability, serviceability or functionality.  
//*
//* This sample may include the names of individuals, companies, brands 
//* and products in order to illustrate concepts as completely as 
//* possible.  All of these names
//* are fictitious and any similarity to the names and addresses used by 
//* actual persons 
//* or business enterprises is entirely coincidental.
//*---------------------------------------------------------------------
//*


////////////////////////////////////////////////////////////////////////////////
//
// Change History:
//
// YYMMDD  	WHO    Description
// --------------------------------------------------------------------
// 020527  	Anand
//
////////////////////////////////////////////////////////////////////////////////


import com.ibm.commerce.datatype.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.ras.*;
import java.math.*;
import java.rmi.*;
import java.sql.*;
import java.util.*;
import javax.ejb.*;
import javax.naming.*;
import com.ibm.commerce.messaging.commands.*;
import com.ibm.eis.sap.jca.generated.*;
import com.ibm.commerce.sap.jca.util.*;
import com.ibm.commerce.orderstatus.objects.*;
import com.ibm.commerce.orderstatus.objimpl.*;
import com.ibm.commerce.util.ResourceBundleHandler;
import com.ibm.commerce.orderstatus.objects.*;

import com.sap.rfc.*;
import com.sap.rfc.exception.*;
import com.ibm.connector2.sap.*;
import com.ibm.commerce.sap.jca.commands.*;


/**
*This command is 
*retreives the status information for a given order number (Sales document).
*sets the retreived status information to a typed property
*execute the Order Status command by providing the above typed
*property as input to Order Status command
*/

public class SAPStatusOrderCmdImpl extends TaskCommandImpl implements SAPStatusOrderCmd {
	public final static String CLASSNAME = "com.ibm.commerce.sap.jca.commands.SAPStatusOrderCmdImpl";
	public static final String COPYRIGHT			= com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
	
	private static final String VALIDATE_PARAMETERS	= "validateParameters";
	private static final String PERFORM_EXECUTE		= "performExecute";
	private SAPConnectionSpec connSpec =null;
	private SAPConnectionFactory connFactory = null;
	private SAPConnection connection = null;

	private String strBackendOrderId = null;
	private String strOrderId = null;
		
	OrderStatusBapi cmd = new OrderStatusBapi();
	private com.ibm.commerce.datatype.TypedProperty requestProperties;
	private com.ibm.commerce.datatype.TypedProperty responseProperties;
/**
 * StatusOrderBapiCmdImpl constructor comment.
 */
public SAPStatusOrderCmdImpl() {
	super();
}
	/**
	 * Returns the orderId.
	 */
	public String getOrderId() { 
		return strOrderId; 
		}

public com.ibm.commerce.datatype.TypedProperty getRequestProperties() {
	return requestProperties;
}

public com.ibm.commerce.datatype.TypedProperty getResponseProperties() {
	return responseProperties;
}
/**
 * Establishing the connection to SAP backend system
 */
 
protected void initialize() throws com.ibm.commerce.exception.ECException {
    final String METHODNAME = "initialize";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);


    try{

        // Setting properties at runtime

	        
            Properties configProp = new Properties();

            configProp.put(
                Context.INITIAL_CONTEXT_FACTORY,               
                PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._INITIAL_CONTEXT_FACTORY));
            configProp.put(Context.PROVIDER_URL, PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._PROVIDER_URL));

            Context ctx = null;
            ctx = new javax.naming.InitialContext(configProp);

			String jndiResource = PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._JNDI_RESOURCE);
	
            com.ibm.connector2.sap.SAPConnectionFactory scf =
                (com.ibm.connector2.sap.SAPConnectionFactory) ctx.lookup(jndiResource);

            com.ibm.connector2.sap.SAPConnectionSpec scs =
                new com.ibm.connector2.sap.SAPConnectionSpec();

  				scs.setUserName(PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._USERID));
				scs.setPassword(PropertiesKeysForSAP.sapResourceBundle.getString(PropertiesKeysForSAP._PASSWD));
				 
            com.ibm.connector2.sap.SAPConnection connection =
                (com.ibm.connector2.sap.SAPConnection) scf.getConnection(scs);
            com.ibm.connector2.sap.SAPInteractionSpec ispec =
                new com.ibm.connector2.sap.SAPInteractionSpec();
            com.ibm.connector2.sap.SAPInteraction interaction =
                (com.ibm.connector2.sap.SAPInteraction) connection.createInteraction();

            cmd.setCeInteractionSpec(ispec);
            cmd.setInteraction(interaction);


     } catch (javax.resource.ResourceException e) {
        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASSNAME,
            METHODNAME, 
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);      
    } catch (javax.naming.NamingException e) {
        throw new ECSystemException(
            ECMessage._ERR_GENERIC,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,true);
    } catch (Exception e) {
        throw new ECSystemException(ECMessage._ERR_GENERIC, CLASSNAME, METHODNAME, e,true);
    }

    ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);
  
	
	}
/**
 * Contains the actual business logic of the command.
 */
 
 
public void performExecute() throws ECException{

    final String METHODNAME = "performExecute";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);
  
	TypedProperty orderStatusProperties = new TypedProperty();
	//retreiveOrderStatusInfo gets detailed order status from SAP, sets status info into typed property
	orderStatusProperties = retreiveOrderStatusInfo();

	
    //invoking Order Status command to update WC with latest order status info from backend 

    OrderStatusCmd updateOrderStatus = (OrderStatusCmd) CommandFactory.createCommand(OrderStatusCmd.NAME, commandContext.getStoreId());
    updateOrderStatus.setAccCheck(false);//to allow the command to be invoked for this user
 	updateOrderStatus.setCommandContext(getCommandContext());
 	orderStatusProperties.put("URL", "/"); 
 	updateOrderStatus.setRequestProperties(orderStatusProperties);
 	updateOrderStatus.execute();
 	  						
    ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);


	}
/**
 * Prepares the data to invoke Order Status command. 
 */
private TypedProperty populateOrderStatusInfo(BAPI_SALESORDER_GETSTATUS bsc)
    throws ECException {
    final String METHODNAME = "populateOrderStatusInfo";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);

    TypedProperty statusInfoProperties = new TypedProperty();

    try {
        BapireturnStructure returnStruct = bsc.getReturn();

        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASSNAME,
            METHODNAME,
            "**SAP Return Type** " + returnStruct.getType());
        ECTrace.trace(
            ECTraceIdentifiers.COMPONENT_EXTERN,
            CLASSNAME,
            METHODNAME,
            "**SAP Return Data** " + returnStruct.getMessage());

        if (returnStruct.getType().equals("E")) {
            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASSNAME,
                METHODNAME,
                "**Failed to retrieve STATUS for the SAP order");

            throw new ECApplicationException(
                ECMessageObjectsForSAP._ERR_ORDER_STATUS,
                CLASSNAME,
                METHODNAME,
                true);

        } else {
            //set the retrieved status information to properties

            BapisdstatTable bapiStatTable = bsc.getStatusinfo();
            Vector itemsVector = new Vector();
            double priceTotal = 0;
            double invoiceValue = 0;
            int rowCount = bapiStatTable.getRowCount();

            if (rowCount > 0) {
                if (bapiStatTable.getRow(0).getDocNumber() != null)
                    statusInfoProperties.putUrlParam(
                        OrderFulfillmentStatusConstants.MERCHANT_ORDER_NUMBER,
                        bapiStatTable.getRow(0).getDocNumber());
                statusInfoProperties.putUrlParam(
                    OrderFulfillmentStatusConstants.ORDER_ID,
                    getRequestProperties()
                        .getString(com.ibm.commerce.order.utils.OrderConstants.EC_ORDER_ID)
                        .toString());
            }

            //populating the item vector	
            for (int i = 0; i < rowCount; i++) {

                BapisdstatTableRow row = bapiStatTable.getRow(i);

                TypedProperty itemInfo = new TypedProperty();

                if (row.getItmNumberString() != null)
                    itemInfo.put(
                        OrderFulfillmentItemStatusConstants.MERCHANT_ITEM_NUMBER,
                        row.getItmNumberString());
                if (row.getMaterial() != null)
                    itemInfo.put(
                        OrderFulfillmentItemStatusConstants.PART_NUMBER,
                        row.getMaterial());

                if (row.getDlvQty() != null)
                    itemInfo.put(
                        OrderFulfillmentItemStatusConstants.QUANTITY_SHIPPED,
                        row.getDlvQty().toString());

                if (row.getDlvStatI() != null)
                    itemInfo.put(
                        OrderFulfillmentItemStatusConstants.ORDER_ITEM_STATUS,
                        row.getDlvStatI().toString());

                if (row.getDelivDate() != null) {
                    long itemShipTime = row.getDelivDate().getTime();
                    if (itemShipTime > 0)
                        itemInfo.put(
                            OrderFulfillmentItemStatusConstants.ACTUAL_SHIP_DATE_TIME,
                            new Timestamp(itemShipTime).toString());
                }

                OrderFulfillmentItemStatusAccessBean oISAB =
                    new OrderFulfillmentItemStatusAccessBean();
                oISAB =
                    oISAB.findByMerchantOrderAndItemNumber(
                        row.getDocNumber(),
                        row.getItmNumberString());
                itemInfo.put(
                    OrderFulfillmentItemStatusConstants.ORDER_ITEM_ID,
                    oISAB.getOrderItemId().toString());

                itemsVector.add(itemInfo);

            } //items loop

            if (rowCount > 0) {

                if (bapiStatTable.getRow(0).getDlvStatH() != null)
                    statusInfoProperties.putUrlParam(
                        OrderFulfillmentStatusConstants.ORDER_STATUS,
                        bapiStatTable.getRow(0).getDlvStatH());

                if (itemsVector.elements().hasMoreElements())
                    statusInfoProperties.putUrlParam(
                        OrderFulfillmentStatusConstants.ITEMS_VECTOR,
                        itemsVector);
            }

            //  statusInfoProperties.put(OrderFulfillmentStatusConstants.VERSIONING,"TRUE");  

            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASSNAME,
                METHODNAME,
                "** Order Status Details retrieved from Backend**");
        } //end else

    } catch (JRfcRemoteException e) {
        throw new ECSystemException(
            ECMessageObjectsForSAP._ERR_JRFC_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.toString()),
            e,
            true);

    } catch (RemoteException e) {
        throw new ECSystemException(
            ECMessage._ERR_REMOTE_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (CreateException e) {
        throw new ECSystemException(
            ECMessage._ERR_CREATE_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (NamingException e) {
        throw new ECSystemException(
            ECMessage._ERR_NAMING_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (FinderException e) {
        throw new ECSystemException(
            ECMessage._ERR_FINDER_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    }
    
    ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);
    return statusInfoProperties;
}
/**
 * retreives the status of the order from the backend system
 */
private TypedProperty retreiveOrderStatusInfo() throws ECException {

    final String METHODNAME = "retreiveOrderStatusInfo";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);

    initialize(); //to start application
    TypedProperty statusInfoProperties = new TypedProperty();

    try {

        //to retreive the backend orderId from existing status record
        OrderFulfillmentStatusAccessBean oSAB = new OrderFulfillmentStatusAccessBean();
        oSAB = oSAB.findByOrderId(new Long(strOrderId));

        strBackendOrderId = oSAB.getMerchantOrderNumber();

        //retrieve the Detailed Status for the given order
        BAPI_SALESORDER_GETSTATUS status = new BAPI_SALESORDER_GETSTATUS();
        status.setSalesdocument(strBackendOrderId);
        cmd.setCeInput(status);

        cmd.execute();


        if (cmd.getCeOutput0().getReturn().getType().equals("E")) {
            ECTrace.trace(
                ECTraceIdentifiers.COMPONENT_EXTERN,
                CLASSNAME,
                METHODNAME,
                "**Failed to retrieve STATUS for the SAP order");

            throw new ECApplicationException(
                ECMessageObjectsForSAP._ERR_ORDER_STATUS,
                CLASSNAME,
                METHODNAME,
                true);

        } else {

            statusInfoProperties = populateOrderStatusInfo(cmd.getCeOutput0());

        }

    } catch (RemoteException e) {
        throw new ECSystemException(
            ECMessage._ERR_REMOTE_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (CreateException e) {
        throw new ECSystemException(
            ECMessage._ERR_CREATE_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (NamingException e) {
        throw new ECSystemException(
            ECMessage._ERR_NAMING_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    } catch (FinderException e) {
        throw new ECSystemException(
            ECMessage._ERR_FINDER_EXCEPTION,
            CLASSNAME,
            METHODNAME,
            ECMessageHelper.generateMsgParms(e.getMessage()),
            e,
            true);
    }
    
    ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);
    return statusInfoProperties;
}
/**
 * sets the orderId.
 */
public void setOrderId(String orderId) {
    strOrderId = orderId;
}

public void setRequestProperties(com.ibm.commerce.datatype.TypedProperty newRequestProperties) {
	requestProperties = newRequestProperties;
}

public void setResponseProperties(com.ibm.commerce.datatype.TypedProperty newResponseProperties) {
	responseProperties = newResponseProperties;
}
}
