package com.ibm.commerce.sap.jca.commands;

/*
 *-------------------------------------------------------------------
 * Licensed Materials - Property of IBM
 *
 * 5724-A18
 *
 * (c) Copyright International Business Machines Corporation. yyyy, 2003
 *     All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 *-------------------------------------------------------------------
 */

//*---------------------------------------------------------------------
//* The sample contained herein is provided to you "AS IS".
//*
//* It is furnished by IBM as a simple example and has not been  
//* thoroughly tested
//* under all conditions.  IBM, therefore, cannot guarantee its 
//* reliability, serviceability or functionality.  
//*
//* This sample may include the names of individuals, companies, brands 
//* and products in order to illustrate concepts as completely as 
//* possible.  All of these names
//* are fictitious and any similarity to the names and addresses used by 
//* actual persons 
//* or business enterprises is entirely coincidental.
//*---------------------------------------------------------------------
//*


////////////////////////////////////////////////////////////////////////////////
//
// Change History:
//
// YYMMDD  	WHO    Description
// --------------------------------------------------------------------
// 020527  	Anand
//
////////////////////////////////////////////////////////////////////////////////

import java.util.*;
import java.rmi.*;
import java.sql.Timestamp;
import java.math.BigDecimal;
import javax.ejb.*;
import javax.naming.*;
import java.lang.*;

import com.ibm.commerce.ras.*;
import com.ibm.commerce.exception.*; 
import com.ibm.commerce.command.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.datatype.*;
import com.ibm.commerce.orderstatus.objimpl.*;

import com.ibm.commerce.sap.jca.util.*;
import com.ibm.commerce.util.ResourceBundleHandler;

public class SAPTrackOrderStatusCmdImpl extends com.ibm.commerce.command.ControllerCommandImpl implements SAPTrackOrderStatusCmd 
{
	/**
	 * This is a controller command which can be invoked to 
	 * check the order status in backend
	 * This inturn invokes StatusOrderBapiCmd task command to
	 * retreive the status information and update the WC tables
	 */
	 
	public final static String CLASSNAME = "com.ibm.commerce.sap.jca.commands.TrackSapOrderStatusCmdImpl"; 
	public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;

	
	protected Long iOrderId = null;
	protected String iMerchantOrderNumber = null;
/**
 * TrackSapOrderStatusCmdImpl constructor comment.
 */
public SAPTrackOrderStatusCmdImpl() {
	super();
}
/**
 * Contains the actual business logic of the command.
 */
 
public void performExecute() throws ECException
	{
		
		final String METHODNAME = "performExecute";
        ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);

        super.performExecute();
				
		String strOrderId = requestProperties.getString(com.ibm.commerce.order.utils.OrderConstants.EC_ORDER_ID, null);
		String strBackendOrderId = requestProperties.getString(OrderFulfillmentStatusConstants.MERCHANT_ORDER_NUMBER, null);
	
  	    //Execute Order Status BAPI Command
		SAPStatusOrderCmd cmd = (SAPStatusOrderCmd)CommandFactory.createCommand(SAPStatusOrderCmd.NAME,commandContext.getStoreId());

		cmd.setCommandContext(commandContext);
		cmd.setOrderId(iOrderId.toString());
		cmd.setRequestProperties(requestProperties);
		cmd.setResponseProperties(responseProperties);
		cmd.execute();
		
		requestProperties = cmd.getRequestProperties();
		responseProperties = cmd.getResponseProperties();

		if (responseProperties == null)
		   responseProperties = new TypedProperty();
				
		String redirectionURL = requestProperties.getString(ECConstants.EC_REDIRECTURL, null);
		
		if (redirectionURL == null)
		   {
			redirectionURL = "OrderDetail";
		   }
		 
  		responseProperties.put(ECConstants.EC_VIEWTASKNAME, redirectionURL);
  				
        ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, CLASSNAME, METHODNAME);
		
	}

public void setRequestProperties(TypedProperty aRequestProperties)
{
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_EXTERN, this.getClass().getName(), "setRequestProperties");

		requestProperties = aRequestProperties;
		
		iOrderId = requestProperties.getLong(OrderFulfillmentStatusConstants.ORDER_ID, null);
			
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_EXTERN, this.getClass().getName(), "setRequestProperties");
}
}
