<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:PasswordEncryption="class:PasswordEncryption"  version="1.0">

<!-- XSL Rule file to transform extracted Data from R/3 to WCS -->


<!-- Variable Name Definations Section Starts -->
  
  <xsl:variable name="RootElement">import</xsl:variable> 
  <xsl:variable name="VersionElement">V5.1</xsl:variable> 
  <xsl:variable name="RootPrefix">@</xsl:variable>
  <xsl:variable name="RecordSeperator">&#13;</xsl:variable>   
  <xsl:variable name="FalseFlag">N</xsl:variable>
  <xsl:variable name="AddrPrefix">Address</xsl:variable>
  <xsl:variable name="AddrBookPrefix">AddressBook</xsl:variable>


  <xsl:variable name="ZeroValue">0</xsl:variable>
  <xsl:variable name="OneValue">1</xsl:variable>
  <xsl:variable name="TwoValue">2</xsl:variable>
  <xsl:variable name="MinusOneValue">-1</xsl:variable>
  <xsl:variable name="DefaultCurrency">USD</xsl:variable>


  <xsl:variable name="DisplayName">Sap Store use only</xsl:variable>
  <xsl:variable name="RegisterType">R</xsl:variable>
  <xsl:variable name="OrgnProfileType">B</xsl:variable>
  <xsl:variable name="UserProfileType">C</xsl:variable>
  <xsl:variable name="CommaSeperator">,</xsl:variable>

  <xsl:variable name="UserType">U</xsl:variable>
  <xsl:variable name="OrgnType">O</xsl:variable>

  <xsl:variable name="Englishid">en_US</xsl:variable>
  <xsl:variable name="Germanid">de_DE</xsl:variable>
  <xsl:variable name="Chineseid">zh_CN</xsl:variable>
  <xsl:variable name="Thaiid">zh_TW</xsl:variable>
  <xsl:variable name="Koreanid">ko_KR</xsl:variable>
  <xsl:variable name="Frenchid">fr_FR</xsl:variable>
  <xsl:variable name="Italianid">it_IT</xsl:variable>
  <xsl:variable name="Japaneseid">ja_JP</xsl:variable>
  <xsl:variable name="Portugueseid">pt_BR</xsl:variable>
  <xsl:variable name="Spanishid">es_ES</xsl:variable>

  <xsl:variable name="RootOrgnName">Root Organization</xsl:variable>
  <xsl:variable name="DefaultOrgnName">Default Organization</xsl:variable>
  <xsl:variable name="OrgnPrefix">o=</xsl:variable>
  <xsl:variable name="UserIdPrefix">uid=</xsl:variable>
  <xsl:variable name="UserAddressType">SB</xsl:variable>
  <xsl:variable name="Status">P</xsl:variable>



  <xsl:param name="InstanceXMLPath" select="'demo.xml'"/>   
  <xsl:param name="ImportLocation" select="'import.dtd'"/>   
  <xsl:param name="MemberIdentifier" select="-2000"/>   
  <xsl:param name="RootOrgnIdentifier" select="-2001"/>   

  <xsl:param name="DefaultUserIdentifier" select="'DefaultUser'"/>   
  <xsl:param name="MerchantKey" select="'7c91de760ccd'"/>   

 <!-- Variable Name Definitions Section Ends -->


<!-- Transformation Rules Section Starts -->
<xsl:template match="/"> 
<xsl:text disable-output-escaping = "yes">&lt;!DOCTYPE import SYSTEM "</xsl:text> 
<xsl:value-of select = "$ImportLocation"/> 
<xsl:text disable-output-escaping = "yes">"&gt;&#13;</xsl:text> 
<xsl:element name="{$RootElement}">
	<xsl:for-each select=".//member">
		 <xsl:call-template name="CreateinMbr"/>
	</xsl:for-each>
	<xsl:for-each select=".//orgentity">
		<xsl:if test="@profiletype='Z1'">
		 	<xsl:call-template name="CreateinOrgentity"/>
		</xsl:if>
	</xsl:for-each>
	<xsl:for-each select=".//users">
		 <xsl:call-template name="CreateinUsers"/>
	</xsl:for-each>
	<xsl:for-each select=".//userreg">
		 <xsl:call-template name="Create"/>
	</xsl:for-each>
	<xsl:for-each select=".//addrbook">
		 <xsl:call-template name="CreateinAddrBook"/>
	</xsl:for-each>
	<xsl:for-each select=".//address">
		 <xsl:call-template name="CreateinAddress"/>
	</xsl:for-each>
	<xsl:for-each select=".//userprof">
		 <xsl:call-template name="CreateinUserProf"/>
	</xsl:for-each>
	<xsl:for-each select=".//userdemo">
		 <xsl:call-template name="CreateinUserDemo"/>
	</xsl:for-each>
	<xsl:for-each select=".//mbrrel">
		 <xsl:call-template name="CreateinMbrRel"/>
	</xsl:for-each>
	<xsl:for-each select=".//busprof">
		<xsl:if test="@profiletype='Z1'">
		 	<xsl:call-template name="CreateinBusProf"/>
		</xsl:if>
	</xsl:for-each>

</xsl:element>	


</xsl:template>


<!-- Transformation Rules Section Ends -->

<!-- Transformation Rules for creating in USERS Starts -->


<xsl:template name="CreateinUsers">   
	<xsl:element name="users">      
              <xsl:attribute name="users_id">
			<xsl:choose>	      
				<xsl:when test="normalize-space(@profiletype)='Z1'">
	                  	<xsl:value-of select="$RootPrefix"/>
     	                  	<xsl:value-of select="$DefaultUserIdentifier"/>
					<xsl:value-of select="@field1"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$RootPrefix"/>
     	                  	<xsl:value-of select="@field1"/>
				</xsl:otherwise>
			</xsl:choose>
	        </xsl:attribute>
		  <xsl:attribute name="dn">
			<xsl:choose>	      
				<xsl:when test="normalize-space(@profiletype)='Z1'">
					<xsl:value-of select="$UserIdPrefix"/>
					<xsl:value-of select="@field1"/>
					<xsl:value-of select="$CommaSeperator"/>
					<xsl:value-of select="$OrgnPrefix"/>
					<xsl:value-of select="normalize-space(@orgnname)"/>
					<xsl:value-of select="$CommaSeperator"/>
					<xsl:value-of select="$OrgnPrefix"/>
	                  	<xsl:value-of select="$RootOrgnName"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$UserIdPrefix"/>
					<xsl:value-of select="@field1"/>
					<xsl:value-of select="$CommaSeperator"/>
					<xsl:value-of select="$OrgnPrefix"/>
					<xsl:value-of select="$DefaultOrgnName"/>
					<xsl:value-of select="$CommaSeperator"/>
					<xsl:value-of select="$OrgnPrefix"/>
	                  	<xsl:value-of select="$RootOrgnName"/>
				</xsl:otherwise>
			</xsl:choose>
              </xsl:attribute>

		 <xsl:attribute name="registertype">
	                  <xsl:value-of select="$RegisterType"/>
              </xsl:attribute>
		<xsl:attribute name="profiletype">
			<xsl:choose>	      
				<xsl:when test="normalize-space(@profiletype)='Z1'">
	                  	<xsl:value-of select="$OrgnProfileType"/>
				</xsl:when>
				<xsl:otherwise>
	                  	<xsl:value-of select="$UserProfileType"/>
				</xsl:otherwise>
			</xsl:choose>
              </xsl:attribute>
		<xsl:attribute name="language_id">
				<xsl:value-of select="$RootPrefix"/>
			<xsl:choose>
				<xsl:when test="normalize-space(@preferredlanguage)='English'">
					<xsl:value-of select="$Englishid"/>
				</xsl:when>
				<xsl:when test="normalize-space(@preferredlanguage)='German'">
					<xsl:value-of select="$Germanid"/>
				</xsl:when>
				<xsl:when test="normalize-space(@preferredlanguage)='Chinese'">
					<xsl:value-of select="$Chineseid"/>
				</xsl:when>
				<xsl:when test="normalize-space(@preferredlanguage)='Thai'">
					<xsl:value-of select="$Thaiid"/>
				</xsl:when>
				<xsl:when test="normalize-space(@preferredlanguage)='Korean'">
					<xsl:value-of select="$Koreanid"/>
				</xsl:when>
				<xsl:when test="normalize-space(@preferredlanguage)='French'">
					<xsl:value-of select="$Frenchid"/>
				</xsl:when>
				<xsl:when test="normalize-space(@preferredlanguage)='Italian'">
					<xsl:value-of select="$Italianid"/>
				</xsl:when>
				<xsl:when test="normalize-space(@preferredlanguage)='Japanese'">
					<xsl:value-of select="$Japaneseid"/>
				</xsl:when>
				<xsl:when test="normalize-space(@preferredlanguage)='Portuguese'">
					<xsl:value-of select="$Portugueseid"/>
				</xsl:when>
				<xsl:when test="normalize-space(@preferredlanguage)='Spanish'">
					<xsl:value-of select="$Spanishid"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$Englishid"/>
				</xsl:otherwise>
			</xsl:choose>
			</xsl:attribute>
		<xsl:attribute name="setccurr">
			<xsl:choose>
				<xsl:when test="normalize-space(@preferredcurrency)=''">
					<xsl:value-of select="$DefaultCurrency"/>
				</xsl:when>
				<xsl:otherwise>
	                  	<xsl:value-of select="normalize-space(@preferredcurrency)"/>
				</xsl:otherwise>
			</xsl:choose>
              </xsl:attribute>
		<xsl:if test="@profiletype!='Z1'">
			<xsl:copy-of select="@field1"/>
		</xsl:if>

 	</xsl:element> 
</xsl:template>

<!-- Transformation Rules for creating in USERS Ends -->

<!-- Transformation Rules for creating in ADDRBOOK Starts -->

<xsl:template name="CreateinAddrBook">   
	<xsl:element name="addrbook">      
              <xsl:attribute name="addrbook_id">
			<xsl:choose>	      
				<xsl:when test="normalize-space(@profiletype)='Z1'">
	                  	<xsl:value-of select="$RootPrefix"/>
     	                  	<xsl:value-of select="$DefaultUserIdentifier"/>
     	      		      <xsl:value-of select="@addrbook_id"/>					
				</xsl:when>
				<xsl:otherwise>
		            	<xsl:value-of select="$RootPrefix"/>
     	      		      <xsl:value-of select="@addrbook_id"/>
				</xsl:otherwise>
			</xsl:choose>
              </xsl:attribute>
		  <xsl:attribute name="displayname">
	                  <xsl:value-of select="$DisplayName"/>
              </xsl:attribute>
		<xsl:attribute name="member_id">
			<xsl:choose>	      
				<xsl:when test="normalize-space(@profiletype)='Z1'">
	                  	<xsl:value-of select="$RootPrefix"/>
     	                  	<xsl:value-of select="$DefaultUserIdentifier"/>
     	      		      <xsl:value-of select="@addrbook_id"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$RootPrefix"/>
	                  	<xsl:value-of select="@addrbook_id"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:attribute>

     </xsl:element> 
	<xsl:if test="@profiletype='Z1'">
	 	<xsl:call-template name="CreateOrgninAddrBook"/>
	</xsl:if>

</xsl:template>

<!-- Transformation Rules for creating in ADDRBOOK Ends -->

<!-- Transformation Rules for creating Organization in ADDRBOOK Starts -->

<xsl:template name="CreateOrgninAddrBook">   
	<xsl:element name="addrbook">      
              <xsl:attribute name="addrbook_id">
			<xsl:value-of select="$RootPrefix"/>
     	      	<xsl:value-of select="@addrbook_id"/>
		  </xsl:attribute>
		  <xsl:attribute name="displayname">
	                  <xsl:value-of select="$DisplayName"/>
              </xsl:attribute>
		<xsl:attribute name="member_id">
			<xsl:value-of select="$RootPrefix"/>
	           	<xsl:value-of select="@addrbook_id"/>
		</xsl:attribute>

     </xsl:element> 
</xsl:template>

<!-- Transformation Rules for creating Organization in ADDRBOOK Ends -->

<!-- Transformation Rules for creating in ADDRESS Starts -->

<xsl:template name="CreateinAddress">   
	<xsl:element name="address">      
              <xsl:attribute name="address_id">
			<xsl:choose>	      
				<xsl:when test="normalize-space(@profiletype)='Z1'">
	                  	<xsl:value-of select="$RootPrefix"/>
     	                  	<xsl:value-of select="$DefaultUserIdentifier"/>
     	      		      <xsl:value-of select="@address_id"/>
	                  	<xsl:value-of select="$RootPrefix"/>
 	                  	<xsl:value-of select="$DefaultUserIdentifier"/> 
     	      		      <xsl:value-of select="@address_id"/>
				</xsl:when>
				<xsl:otherwise>
		            	<xsl:value-of select="$RootPrefix"/>
     	      		      <xsl:value-of select="@address_id"/>
		            	<xsl:value-of select="$RootPrefix"/>
     	      		      <xsl:value-of select="@address_id"/>
				</xsl:otherwise>
			</xsl:choose>
              </xsl:attribute>
		  <xsl:attribute name="addrbook_id">
			<xsl:choose>	      
				<xsl:when test="normalize-space(@profiletype)='Z1'">
	                  	<xsl:value-of select="$RootPrefix"/>
     	                  	<xsl:value-of select="$DefaultUserIdentifier"/>
     	      		      <xsl:value-of select="@address_id"/>
				</xsl:when>
				<xsl:otherwise>
		            	<xsl:value-of select="$RootPrefix"/>
     	      		      <xsl:value-of select="@address_id"/>
				</xsl:otherwise>
			</xsl:choose>
              </xsl:attribute>
		  <xsl:attribute name="nickname">
	                 <xsl:value-of select="normalize-space(@address_id)"/>
              </xsl:attribute>
		  <xsl:attribute name="member_id">
			<xsl:choose>	      
				<xsl:when test="normalize-space(@profiletype)='Z1'">
	                  	<xsl:value-of select="$RootPrefix"/>
     	                  	<xsl:value-of select="$DefaultUserIdentifier"/>
     	      		      <xsl:value-of select="@address_id"/>
				</xsl:when>
				<xsl:otherwise>
		            	<xsl:value-of select="$RootPrefix"/>
     	      		      <xsl:value-of select="@address_id"/>
				</xsl:otherwise>
			</xsl:choose>
              </xsl:attribute>
		  <xsl:attribute name="address1">
	                 <xsl:value-of select="normalize-space(@addressline)"/>
              </xsl:attribute>
		  <xsl:attribute name="city">
	                 <xsl:value-of select="normalize-space(@city)"/>
              </xsl:attribute>
		<xsl:attribute name="country">
	                 <xsl:value-of select="normalize-space(@country)"/>
              </xsl:attribute>
		<xsl:attribute name="fax1">
	                 <xsl:value-of select="normalize-space(@fax)"/>
              </xsl:attribute>
		<xsl:attribute name="lastname">
	                 <xsl:value-of select="normalize-space(@lastname)"/>
              </xsl:attribute>
		<xsl:attribute name="middlename">
	                 <xsl:value-of select="normalize-space(@middlename)"/>
              </xsl:attribute>
		<xsl:attribute name="firstname">
	                 <xsl:value-of select="normalize-space(@firstname)"/>
              </xsl:attribute>
		<xsl:attribute name="phone1">
	                 <xsl:value-of select="normalize-space(@telephone1)"/>
              </xsl:attribute>
		<xsl:attribute name="phone2">
	                 <xsl:value-of select="normalize-space(@telephone2)"/>
              </xsl:attribute>
		<xsl:attribute name="zipcode">
	                 <xsl:value-of select="normalize-space(@zipcode)"/>
              </xsl:attribute>
		<xsl:attribute name="state">
	                 <xsl:value-of select="normalize-space(@state)"/>
              </xsl:attribute>
		<xsl:attribute name="businesstitle">
	                 <xsl:value-of select="normalize-space(@title)"/>
            </xsl:attribute>
		<xsl:attribute name="addresstype">
	                 <xsl:value-of select="$UserAddressType"/>
            </xsl:attribute>
		<xsl:attribute name="isprimary">
	                 <xsl:value-of select="$OneValue"/>
            </xsl:attribute>
		<xsl:attribute name="status">
	                 <xsl:value-of select="$Status"/>
            </xsl:attribute>
		<xsl:attribute name="selfaddress">
	                 <xsl:value-of select="$OneValue"/>
            </xsl:attribute>
     </xsl:element> 
	<xsl:if test="@profiletype='Z1'">
	 	<xsl:call-template name="CreateOrgninAddress"/>
	</xsl:if>

</xsl:template>

<!-- Transformation Rules for creating in ADDRESS Ends -->

<!-- Transformation Rules for creating Organization in ADDRESS Starts -->

<xsl:template name="CreateOrgninAddress">   
	<xsl:element name="address">      
              <xsl:attribute name="address_id">
            	<xsl:value-of select="$RootPrefix"/>
     		      <xsl:value-of select="@address_id"/>
            	<xsl:value-of select="$RootPrefix"/>
     		      <xsl:value-of select="@address_id"/>
              </xsl:attribute>
		  <xsl:attribute name="addrbook_id">
            	<xsl:value-of select="$RootPrefix"/>
     		      <xsl:value-of select="@address_id"/>
              </xsl:attribute>
		  <xsl:attribute name="nickname">
                 <xsl:value-of select="normalize-space(@lastname)"/>
              </xsl:attribute>
		  <xsl:attribute name="member_id">
            	<xsl:value-of select="$RootPrefix"/>
     		      <xsl:value-of select="@address_id"/>
              </xsl:attribute>
		  <xsl:attribute name="address1">
	                 <xsl:value-of select="normalize-space(@addressline)"/>
              </xsl:attribute>
		  <xsl:attribute name="city">
	                 <xsl:value-of select="normalize-space(@city)"/>
              </xsl:attribute>
		  <xsl:attribute name="country">
	                 <xsl:value-of select="normalize-space(@country)"/>
              </xsl:attribute>
		  <xsl:attribute name="fax1">
	                 <xsl:value-of select="normalize-space(@fax)"/>
              </xsl:attribute>
		  <xsl:attribute name="lastname">
	                 <xsl:value-of select="normalize-space(@lastname)"/>
              </xsl:attribute>
		  <xsl:attribute name="middlename">
	                 <xsl:value-of select="normalize-space(@middlename)"/>
              </xsl:attribute>
		  <xsl:attribute name="firstname">
	                 <xsl:value-of select="normalize-space(@firstname)"/>
              </xsl:attribute>
		  <xsl:attribute name="phone1">
	                 <xsl:value-of select="normalize-space(@telephone1)"/>
              </xsl:attribute>
		  <xsl:attribute name="phone2">
	                 <xsl:value-of select="normalize-space(@telephone2)"/>
              </xsl:attribute>
		  <xsl:attribute name="zipcode">
	                 <xsl:value-of select="normalize-space(@zipcode)"/>
              </xsl:attribute>
		  <xsl:attribute name="state">
	                 <xsl:value-of select="normalize-space(@state)"/>
              </xsl:attribute>
		  <xsl:attribute name="businesstitle">
	                 <xsl:value-of select="normalize-space(@title)"/>
              </xsl:attribute>
		<xsl:attribute name="status">
	                 <xsl:value-of select="$Status"/>
            </xsl:attribute>
		<xsl:attribute name="selfaddress">
	                 <xsl:value-of select="$OneValue"/>
            </xsl:attribute>
     </xsl:element> 
</xsl:template>

<!-- Transformation Rules for creating Organization in ADDRESS Ends -->

<!-- Transformation Rules for creating in USERREG Starts -->

<xsl:template name="Create">   
	<xsl:element name="userreg">      
              <xsl:attribute name="users_id">
			<xsl:choose>	      
				<xsl:when test="normalize-space(@profiletype)='Z1'">
	                  	<xsl:value-of select="$RootPrefix"/>
     	                  	<xsl:value-of select="$DefaultUserIdentifier"/>
     	      		      <xsl:value-of select="@logonid"/>
				</xsl:when>
				<xsl:otherwise>
		            	<xsl:value-of select="$RootPrefix"/>
     	      		      <xsl:value-of select="@logonid"/>
				</xsl:otherwise>
			</xsl:choose>
              </xsl:attribute>
		 <xsl:attribute name="passwordexpired">
	                  <xsl:value-of select="$OneValue"/>
              </xsl:attribute>
		<xsl:attribute name="status">
	                  <xsl:value-of select="$OneValue"/>
             </xsl:attribute>
		
		<xsl:variable name="TempSaltVariable">
			<xsl:value-of select="PasswordEncryption:generateSaltKey()"/>
		</xsl:variable>
	
		<xsl:attribute name="salt">
	                <xsl:value-of select="$TempSaltVariable"/>
             </xsl:attribute>

		<xsl:attribute name="logonpassword">
			<xsl:value-of select="PasswordEncryption:encryptPassword(string(normalize-space(@password)),string($MerchantKey),string($TempSaltVariable))"/>  

	      </xsl:attribute>
		<xsl:attribute name="plcyacct_id">
	                 <xsl:value-of select="$MinusOneValue"/>
            </xsl:attribute>

		<xsl:copy-of select="@logonid "/>
     </xsl:element> 
</xsl:template>

<!-- Transformation Rules for creating in USERREG Ends -->

<!-- Transformation Rules for creating in USERPROF Starts -->

<xsl:template name="CreateinUserProf">   
	<xsl:element name="userprof">      
              <xsl:attribute name="users_id">
			<xsl:choose>	      
				<xsl:when test="normalize-space(@profiletype)='Z1'">
	                  	<xsl:value-of select="$RootPrefix"/>
     	                  	<xsl:value-of select="$DefaultUserIdentifier"/>
     	      		      <xsl:value-of select="@users_id"/>
				</xsl:when>
				<xsl:otherwise>
		            	<xsl:value-of select="$RootPrefix"/>
     	      		      <xsl:value-of select="@users_id"/>
				</xsl:otherwise>
			</xsl:choose>
              </xsl:attribute>
		<xsl:attribute name="preferreddelivery">
	                  <xsl:value-of select="normalize-space(@preferreddelivery)"/>
              </xsl:attribute>
     </xsl:element> 
</xsl:template>

<!-- Transformation Rules for creating in USERPROF Ends -->

<!-- Transformation Rules for creating in USERDEMO Starts -->

<xsl:template name="CreateinUserDemo">   
	<xsl:element name="userdemo">      
              <xsl:attribute name="users_id">
			<xsl:choose>	      
				<xsl:when test="normalize-space(@profiletype)='Z1'">
	                  	<xsl:value-of select="$RootPrefix"/>
     	                  	<xsl:value-of select="$DefaultUserIdentifier"/>
     	      		      <xsl:value-of select="@users_id"/>
				</xsl:when>
				<xsl:otherwise>
		            	<xsl:value-of select="$RootPrefix"/>
     	      		      <xsl:value-of select="@users_id"/>
				</xsl:otherwise>
			</xsl:choose>
              </xsl:attribute>
     </xsl:element> 
</xsl:template>

<!-- Transformation Rules for creating in USERDEMO Ends -->

<!-- Transformation Rules for creating in MEMBER Starts -->

<xsl:template name="CreateinMbr">   
	<xsl:element name="member">      
              <xsl:attribute name="member_id">
				<xsl:value-of select="$RootPrefix"/>
	                  <xsl:value-of select="@member_id"/>
              </xsl:attribute>
		 <xsl:attribute name="type">
			<xsl:choose>	      
				<xsl:when test="normalize-space(@profiletype)='Z1'">
	                  	<xsl:value-of select="$OrgnType"/>
				</xsl:when>
				<xsl:otherwise>
	                  	<xsl:value-of select="$UserType"/>
				</xsl:otherwise>
			</xsl:choose>
              </xsl:attribute>
		  <xsl:attribute name="state">
	                  <xsl:value-of select="$OneValue"/>
              </xsl:attribute>
	</xsl:element> 
	<xsl:if test="@profiletype='Z1'">
	 	<xsl:call-template name="CreateDefaultinMember"/>
	</xsl:if>
</xsl:template>

<!-- Transformation Rules for creating in MEMBER Ends -->

<!-- Transformation Rules for creating default member in MEMBER Starts -->

<xsl:template name="CreateDefaultinMember">   
	<xsl:element name="member">      
              <xsl:attribute name="member_id">
				<xsl:value-of select="$RootPrefix"/>
	                  <xsl:value-of select="$DefaultUserIdentifier"/>
	                  <xsl:value-of select="@member_id"/>
              </xsl:attribute>
		 <xsl:attribute name="type">
		           	<xsl:value-of select="$UserType"/>
             </xsl:attribute>
		 <xsl:attribute name="state">
	                  <xsl:value-of select="$OneValue"/>
             </xsl:attribute>
	</xsl:element> 
</xsl:template>

<!-- Transformation Rules for creating default member in MEMBER Ends -->

<!-- Transformation Rules for creating B2B user in MBRREL Starts -->

<xsl:template name="CreateinMbrRel">   
	<xsl:element name="mbrrel">
         <xsl:attribute name="ancestor_id">
		<xsl:choose>	      
			<xsl:when test="normalize-space(@profiletype)='Z1'">
				<xsl:value-of select="$RootPrefix"/>
	                  <xsl:value-of select="@descendant_id"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$MemberIdentifier"/>
			</xsl:otherwise>
		</xsl:choose>
        </xsl:attribute>
	  <xsl:attribute name="descendant_id">
		<xsl:choose>	      
			<xsl:when test="normalize-space(@profiletype)='Z1'">
				<xsl:value-of select="$RootPrefix"/>
	                  <xsl:value-of select="$DefaultUserIdentifier"/>
                  	<xsl:value-of select="@descendant_id"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$RootPrefix"/>
                  	<xsl:value-of select="@descendant_id"/>
			</xsl:otherwise>
		</xsl:choose>

	  </xsl:attribute>
	  <xsl:attribute name="sequence">
			<xsl:value-of select="$OneValue"/>
	  </xsl:attribute>
	</xsl:element> 
	<xsl:choose>	      
   	  <xsl:when test="normalize-space(@profiletype)='Z1'">
	 	<xsl:call-template name="CreateOrgninMbrRel"/>
	 	<xsl:call-template name="CreateUserinMbrRel"/>
	  </xsl:when>
	  <xsl:otherwise>
	 	<xsl:call-template name="CreateUserinMbrRel"/>
	  </xsl:otherwise>
	</xsl:choose>
</xsl:template>

<!-- Transformation Rules for creating B2B user in MBRREL Ends -->

<!-- Transformation Rules for creating B2C user in MBRREL Starts -->


<xsl:template name="CreateUserinMbrRel">   
	<xsl:element name="mbrrel">
        <xsl:attribute name="ancestor_id">
			<xsl:value-of select="$RootOrgnIdentifier"/>
	  </xsl:attribute>
	  <xsl:attribute name="descendant_id">
	  	<xsl:choose>	      
			<xsl:when test="normalize-space(@profiletype)='Z1'">
				<xsl:value-of select="$RootPrefix"/>
	                  <xsl:value-of select="$DefaultUserIdentifier"/>
                  	<xsl:value-of select="@descendant_id"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$RootPrefix"/>
                  	<xsl:value-of select="@descendant_id"/>
			</xsl:otherwise>
		</xsl:choose>
	  </xsl:attribute>
	  <xsl:attribute name="sequence">
			<xsl:value-of select="$TwoValue"/>
	  </xsl:attribute>
	</xsl:element> 
</xsl:template>

<!-- Transformation Rules for creating B2C user in MBRREL Ends -->

<!-- Transformation Rules for creating Organization in MBRREL Starts -->

<xsl:template name="CreateOrgninMbrRel">   
	<xsl:element name="mbrrel">
        <xsl:attribute name="ancestor_id">
			<xsl:value-of select="$RootOrgnIdentifier"/>
	  </xsl:attribute>
	  <xsl:attribute name="descendant_id">
			<xsl:value-of select="$RootPrefix"/>
                 	<xsl:value-of select="@descendant_id"/>
	  </xsl:attribute>
	  <xsl:attribute name="sequence">
			<xsl:value-of select="$OneValue"/>
	  </xsl:attribute>

	</xsl:element> 
</xsl:template>

<!-- Transformation Rules for creating Organization in MBRREL Ends -->

<!-- Transformation Rules for creating Organization in ORGENTITY Starts -->

<xsl:template name="CreateinOrgentity">   
	<xsl:element name="orgentity">
        <xsl:attribute name="orgentity_id">
			<xsl:value-of select="$RootPrefix"/>
			<xsl:value-of select="@field1"/>
	  </xsl:attribute>
	  <xsl:attribute name="orgentityname">
                 	<xsl:value-of select="normalize-space(@orgentityname)"/>
	  </xsl:attribute>
	  <xsl:attribute name="orgentitytype">
                 	<xsl:value-of select="$OrgnType"/>
	  </xsl:attribute>
	
	  <xsl:attribute name="dn">
			<xsl:value-of select="$OrgnPrefix"/>
			<xsl:value-of select="normalize-space(@orgentityname)"/>
			<xsl:value-of select="$CommaSeperator"/>
			<xsl:value-of select="$OrgnPrefix"/>
                 	<xsl:value-of select="$RootOrgnName"/>
	  </xsl:attribute>
	<xsl:copy-of select="@field1"/>
	</xsl:element> 
</xsl:template>

<!-- Transformation Rules for creating Organization in ORGENTITY Ends -->

<!-- Transformation Rules for creating in BUSPROF Starts -->

<xsl:template name="CreateinBusProf">   
	<xsl:element name="busprof">
        <xsl:attribute name="users_id">
			<xsl:value-of select="$RootPrefix"/>
            	<xsl:value-of select="$DefaultUserIdentifier"/>
                 	<xsl:value-of select="@org_id"/>
	  </xsl:attribute>
	  <xsl:attribute name="org_id">
			<xsl:value-of select="$RootPrefix"/>
                 	<xsl:value-of select="@org_id"/>
	  </xsl:attribute>
	</xsl:element> 
</xsl:template>

<!-- Transformation Rules for creating in BUSPROF Ends -->

</xsl:stylesheet>
